#include <math.h>
#include "andre.h"

F1::F1 ( void ) : objective_function ()
{
    int i;
    Dim = 1;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=0.0;
        Domain[i][1]=1.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 1.12323;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "F1" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double F1::value ( double *oCH )
{
    double f;

    f=2*(oCH[0]-0.75)*(oCH[0]-0.75)+sin(5*M_PI*oCH[0]-0.4*M_PI)-0.125;

    return -f;
}

F3::F3 ( void ) : objective_function ()
{
    int i;
    Dim = 1;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-10.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 12.03125;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "F3" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double F3::value ( double *oCH )
{
    int i;
    double f=0.0;

    for ( i=1; i<6; i++ )
    {
        f+=i*sin((i+1)*oCH[0]+i);
    }
    return f;
}

Branin::Branin ( void ) : objective_function ()
{
    int i;
    Dim = 2;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
    }
    Domain[0][0]=-5.0;
    Domain[0][1]=10.0;
    Domain[0][2]=0.0;
    Domain[1][0]=0.0;
    Domain[1][1]=15.0;
    Domain[1][2]=0.0;
    optimum = new double;
    *optimum = -0.39789;
    precision = -0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "Branin" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Branin::value ( double *oCH )
{
    double f;
    double a,b,c,d,h,e;

    a=1;
    b=5.1/(4*M_PI*M_PI);
    c=5/M_PI;
    d=6;
    h=10;
    e=1/(8*M_PI);

    f=a*pow(oCH[1]-b*oCH[0]*oCH[0]+c*oCH[0]-d, 2)+h*(1-e)*cos(oCH[0])+h;

    return -f;
}

Camelback::Camelback ( void ) : objective_function ()
{
    int i;
    Dim = 2;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
    }
    Domain[0][0]=-3.0;
    Domain[0][1]=3.0;
    Domain[0][2]=0.0;
    Domain[1][0]=-2.0;
    Domain[1][1]=2.0;
    Domain[1][2]=0.0;
    optimum = new double;
    *optimum = 1.03163;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "Camelback" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Camelback::value ( double *oCH )
{
    double f;

    f=(4-2.1*oCH[0]*oCH[0]+pow(oCH[0],4)/3)*oCH[0]*oCH[0]+oCH[0]*oCH[1]+(-4+4*oCH[1]*oCH[1])*oCH[1]*oCH[1];

    return -f;
}

Goldprice::Goldprice ( void ) : objective_function ()
{
    int i;
    Dim = 2;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-2.0;
        Domain[i][1]=2.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = -3;
    precision = -0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "Goldprice" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Goldprice::value ( double *oCH )
{
    double f;

    f=1+(oCH[0]+oCH[1]+1)*(oCH[0]+oCH[1]+1)*(19-14*oCH[0]+3*oCH[0]*oCH[0]-14*oCH[1]+6*oCH[0]*oCH[1]+3*oCH[1]*oCH[1]);
    f*=30+(2*oCH[0]-3*oCH[1])*(2*oCH[0]-3*oCH[1])*(18-32*oCH[0]+12*oCH[0]*oCH[0]+48*oCH[1]-36*oCH[0]*oCH[1]+27*oCH[1]*oCH[1]);
    return -f;
}

PShubert1::PShubert1 ( void ) : objective_function ()
{
    int i;
    Dim = 2;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-10.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 186.73091;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "PShubert1" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double PShubert1::value ( double *oCH )
{
    int i;
    double f, fx=0.0, fy=0.0;
    double b=0.5;

    for ( i=1; i<6; i++ )
    {
        fx+=i*cos((i+1)*oCH[0]+i);
        fy+=i*cos((i+1)*oCH[1]+i);
    }

    f=fx*fy+b*((oCH[0]+1.42513)*(oCH[0]+1.42513)+(oCH[1]+0.80032)*(oCH[1]+0.80032));

    return -f;
}

PShubert2::PShubert2 ( void ) : objective_function ()
{
    int i;
    Dim = 2;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-10.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 186.73091;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "PShubert2" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double PShubert2::value ( double *oCH )
{
    int i;
    double f, fx=0.0, fy=0.0;
    double b=1.0;

    for ( i=1; i<6; i++ )
    {
        fx+=i*cos((i+1)*oCH[0]+i);
        fy+=i*cos((i+1)*oCH[1]+i);
    }

    f=fx*fy+b*((oCH[0]+1.42513)*(oCH[0]+1.42513)+(oCH[1]+0.80032)*(oCH[1]+0.80032));

    return -f;
}

Quartic::Quartic ( void ) : objective_function ()
{
    int i;
    Dim = 2;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-10.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 0.35239;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "Quartic" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Quartic::value ( double *oCH )
{
    double f;

    f=pow(oCH[0],4)/4-oCH[0]*oCH[0]/2+oCH[0]/10+oCH[1]*oCH[1]/2;

    return -f;
}

Shubert::Shubert ( void ) : objective_function ()
{
    int i;
    Dim = 2;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-10.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 186.73091;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "Shubert" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Shubert::value ( double *oCH )
{
    int i;
    double f, fx=0.0, fy=0.0;

    for ( i=1; i<6; i++ )
    {
        fx+=i*cos((i+1)*oCH[0]+i);
        fy+=i*cos((i+1)*oCH[1]+i);
    }

    f=fx*fy;

    return -f;
}

Hartman1::Hartman1 ( void ) : objective_function ()
{
    int i;
    Dim = 3;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=0.0;
        Domain[i][1]=1.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 3.86278;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "Hartman1" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Hartman1::value ( double *oCH )
{
    int i, j;
    double f=0.0, x;

    double a[4][3]={{3.0,10.0,30.0},{0.1,10.0,35.0},{3.0,10.0,30.0},{0.1,10.0,35.0}};
    double c[4]={1.0,1.2,3.0,3.2};
    double p[4][3]={{0.36890,0.1170,0.2673},{0.46990,0.4387,0.7470},{0.10910,0.8732,0.5547},{0.03815,0.5743,0.8828}};

    for ( i=0; i<4; i++ )
    {
        x=0.0;
        for ( j=0; j<3; j++ )
        {
            x-=a[i][j]*(oCH[j]-p[i][j])*(oCH[j]-p[i][j]);
        }
        f+=c[i]*exp(x);
    }

    return f;
}

Shekel1::Shekel1 ( void ) : objective_function ()
{
    int i;
    Dim = 4;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=0.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 10.15320;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "Shekel1" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Shekel1::value ( double *oCH )
{
    int i, j, m;
    double f=0.0, fm;

    m=5;
    double a[5][4]={{4.0,4.0,4.0,4.0},{1.0,1.0,1.0,1.0},{8.0,8.0,8.0,8.0},{6.0,6.0,6.0,6.0},{3.0,7.0,3.0,7.0}};
    double c[5]={0.1,0.2,0.2,0.4,0.4};

    for ( i=0; i<m; i++ )
    {
        fm=0.0;
        for ( j=0; j<4; j++ ) fm+=(oCH[j]-a[i][j])*(oCH[j]-a[i][j]);
        f+=1/(fm+c[i]);
    }

    return f;
}

Shekel2::Shekel2 ( void ) : objective_function ()
{
    int i;
    Dim = 4;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=0.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 10.40294;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "Shekel2" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Shekel2::value ( double *oCH )
{
    int i, j, m;
    double f=0.0, fm;

    m=7;
    double a[10][4]={{4.0,4.0,4.0,4.0},{1.0,1.0,1.0,1.0},{8.0,8.0,8.0,8.0},{6.0,6.0,6.0,6.0},{3.0,7.0,3.0,7.0},{2.0,9.0,2.0,9.0},{5.0,5.0,3.0,3.0}};
    double c[10]={0.1,0.2,0.2,0.4,0.4,0.6,0.6};

    for ( i=0; i<m; i++ )
    {
        fm=0.0;
        for ( j=0; j<4; j++ ) fm+=(oCH[j]-a[i][j])*(oCH[j]-a[i][j]);
        f+=1/(fm+c[i]);
    }

    return f;
}

Shekel3::Shekel3 ( void ) : objective_function ()
{
    int i;
    Dim = 4;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=0.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 10.53641;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "Shekel3" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Shekel3::value ( double *oCH )
{
    int i, j, m;
    double f=0.0, fm;

    m=10;
    double a[10][4]={{4.0,4.0,4.0,4.0},{1.0,1.0,1.0,1.0},{8.0,8.0,8.0,8.0},{6.0,6.0,6.0,6.0},{3.0,7.0,3.0,7.0},{2.0,9.0,2.0,9.0},{5.0,5.0,3.0,3.0},{8.0,1.0,8.0,1.0},{6.0,2.0,6.0,2.0},{7.0,3.6,7.0,3.6}};
    double c[10]={0.1,0.2,0.2,0.4,0.4,0.6,0.6,0.7,0.5,0.5};

    for ( i=0; i<m; i++ )
    {
        fm=0.0;
        for ( j=0; j<4; j++ ) fm+=(oCH[j]-a[i][j])*(oCH[j]-a[i][j]);
        f+=1/(fm+c[i]);
    }

    return f;
}

Hartman2::Hartman2 ( void ) : objective_function ()
{
    int i;
    Dim = 6;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=0.0;
        Domain[i][1]=1.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 3.32237;
    precision = 0.01*(*optimum);
    Return_to_domain=1;
    sprintf ( name, "Hartman2" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Hartman2::value ( double *oCH )
{
    int i, j;
    double f=0.0, x;

    double a[4][6]={{10.0,3.0,17.0,3.5,1.7,8.0},{0.05,10.0,17.0,0.1,8.0,14.0},{3.0,3.5,1.7,10.0,17.0,8.0},{17.0,8.0,0.05,10.0,0.01,14.0}};
    double c[4]={1.0,1.2,3.0,3.2};
    double p[4][6]={{0.1312,0.1696,0.5569,0.0124,0.8283,0.5886},{0.2329,0.4135,0.8307,0.3736,0.1004,0.9991},{0.2348,0.1451,0.3522,0.2883,0.3047,0.6650},{0.4047,0.8828,0.8732,0.5743,0.1091,0.0381}};

    for ( i=0; i<4; i++ )
    {
        x=0.0;
        for ( j=0; j<6; j++ )
        {
            x-=a[i][j]*(oCH[j]-p[i][j])*(oCH[j]-p[i][j]);
        }
        f+=c[i]*exp(x);
    }

    return f;
}

Hosc45::Hosc45 ( void ) : objective_function ()
{
    int i;
    Dim = 10;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=0.0;
        Domain[i][1]=double(i+1);
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = -1;
    precision = 0.01;
    Return_to_domain=1;
    sprintf ( name, "Hosc45" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Hosc45::value ( double *oCH )
{
    int i,j;
    double f=1.0;

    for ( i=0; i<Dim; i++ )
    {
        f*=oCH[i];
        f/=i+1;
    }

    return f-2.0;
}

Brown1::Brown1 ( void ) : objective_function ()
{
    int i;
    Dim = 20;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-1.0;
        Domain[i][1]=4.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = -2;
    precision = 0.02;
    Return_to_domain=1;
    sprintf ( name, "Brown1" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Brown1::value ( double *oCH )
{
    int i;
    double f1=0.0, f2=0.0;

    for ( i=0; i<20; i+=2 )
    {
        f1+=oCH[i]-3;
        f2+=0.001*(oCH[i]-3)*(oCH[i]-3)-oCH[i]+oCH[i+1]+exp(20*(oCH[i]-oCH[i+1]));
    }

    return -f1*f1-f2;
}

Brown3::Brown3 ( void ) : objective_function ()
{
    int i;
    Dim = 20;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-1.0;
        Domain[i][1]=4.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 0.0;
    precision = 0.1;
    Return_to_domain=1;
    sprintf ( name, "Brown3" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Brown3::value ( double *oCH )
{
    int i;
    double f=0.0;

    for ( i=0; i<19; i++ )
    {
        f+=pow(oCH[i]*oCH[i],oCH[i+1]*oCH[i+1]+1)+pow(oCH[i+1]*oCH[i+1],oCH[i]*oCH[i]+1);
    }

    return -f;
}

Chainsing::Chainsing ( void ) : objective_function ()
{
    int i;
    Dim = 20;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-1.0;
        Domain[i][1]=4.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 0.0;
    precision = 0.1;
    Return_to_domain=1;
    sprintf ( name, "Chainsing" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double Chainsing::value ( double *oCH )
{
    int i;
    double f1=0.0, f2=0.0;

    for ( i=0; i<20; i+=2 )
    {
        f1+=oCH[i]-3;
        f2+=0.001*(oCH[i]-3)*(oCH[i]-3)-oCH[i]+oCH[i+1]+exp(20*(oCH[i]-oCH[i+1]));
    }

    return -f1*f1-f2;
}

F5n::F5n ( void ) : objective_function ()
{
    int i;
    Dim = 20;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-10.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 0.0;
    precision = 0.1;
    Return_to_domain=1;
    sprintf ( name, "F5n" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double F5n::value ( double *oCH )
{
    int i;
    double f, f1=0.0;

    for ( i=0; i<19; i++ )
    {
        f1+=(oCH[i]-1)*(oCH[i]-1)/16*(1+10*sin(M_PI*(1+(oCH[i+1]-1)/4))*sin(M_PI*(1+(oCH[i+1]-1)/4)));
    }

    f=M_PI/20*(10*sin(M_PI*(1+(oCH[0]-1)/4))*sin(M_PI*(1+(oCH[0]-1)/4))+f1+(oCH[19]-1)*(oCH[19]-1)/16);

    return -f;
}

F10n::F10n ( void ) : objective_function ()
{
    int i;
    Dim = 20;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-10.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 0.0;
    precision = 0.1;
    Return_to_domain=1;
    sprintf ( name, "F10n" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double F10n::value ( double *oCH )
{
    int i;
    double f, f1=0.0;

    for ( i=0; i<19; i++ )
    {
        f1+=(oCH[i]-1)*(oCH[i]-1)*(1+10*sin(M_PI*oCH[i+1])*sin(M_PI*oCH[i+1]));
    }

    f=M_PI/20*(10*sin(M_PI*oCH[0])*sin(M_PI*oCH[0])+f1+(oCH[19]-1)*(oCH[19]-1));

    return -f;
}

F15n::F15n ( void ) : objective_function ()
{
    int i;
    Dim = 20;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-10.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 0.0;
    precision = 0.1;
    Return_to_domain=1;
    sprintf ( name, "F15n" );
    outfile = new char [256];
    outfile = &(name[0]);
}

double F15n::value ( double *oCH )
{
    int i;
    double f, f1=0.0;

    for ( i=0; i<19; i++ )
    {
        f1+=(oCH[i]-1.0)*(oCH[i]-1.0)*(1.0+sin(3.0*M_PI*oCH[i+1])*sin(3.0*M_PI*oCH[i+1]));
    }

    f=1.0/10.0*(sin(3.0*M_PI*oCH[0])*sin(3.0*M_PI*oCH[0])+f1+1.0/10.0*(oCH[19]-1.0)*(oCH[19]-1.0)*(1.0+sin(2.0*M_PI*oCH[19])*sin(2.0*M_PI*oCH[19])));

    return -f;
}




