#include "vilog.c"

#if !defined ( __GARTOU__ )
#include "gartou.h"
#endif

#include <math.h>
#include <stdlib.h>

gartou::gartou ( void )
{
    fitness_calls_limit = 200000;
    vilog = 1;
    vilog_gati = 0;

//------------------variables of SADE:

    pool_rate = 12;

    radioactivity = 0.003;
    local_radioactivity = 0.00;
    mutation_rate = 0.75;
    mutagen_rate = 1226.0;

    cross_rate = 0.365;

    ActualSize=0;
    generation=0;
    fitness_call=0;
    PoolSize=0;
    SelectedSize=0;
    Force=NULL;
    mutagen=NULL;
    CH=NULL;
    bsf=NULL;
    bsf_value=MinDouble;
    btg=NULL;
    btg_value=MinDouble;

//------------------constants and variables of CERAF:

    ExtremeSize = 1048576;
    radius_rate = 0.5;
    disinteg_rate  = 0.995;
    quiet = 1700000;

    n_extreme=0;
    killed=0;
    monsters=0;
    Extremes=NULL;

    id=0;
    lastchange=0;
    lastbtg=MinDouble;
    quiet_generation=0.0;
}

gartou::~gartou ( void )
{
}

// -------------------------------------- SADE TECHNOLOGY --------------------------------------------------

double* gartou::new_point ( void )
{
    int i;
    double *x;

    x = new double [F->Dim];
    for ( i=0; i<F->Dim; i++ )
    {
        x[i] = random_double ( F->Domain[i][0], F->Domain[i][1] );
    }
    return x;
}

void gartou::configuration ( void )
{
    PoolSize = 2*pool_rate*F->Dim;
    SelectedSize = pool_rate*F->Dim;

    int i;
    mutagen = new double [F->Dim];
    for ( i=0; i<F->Dim; i++ )
    {
        mutagen[i] = (F->Domain[i][1]-F->Domain[i][0])/mutagen_rate;
        if ( F->Domain[i][2]>mutagen[i] ) mutagen[i] = F->Domain[i][2];
    }

    quiet_generation=quiet/SelectedSize*F->Dim;
}

void gartou::EVALUATE_GENERATION ( int start )
{
    int i,j, k;

    for ( i=start*SelectedSize; i<ActualSize; i++ )
    {
        if ( F->Return_to_domain )
        {
            for ( k=0; k<F->Dim; k++ )
            {
                if ( CH[i][k]<F->Domain[k][0] ) CH[i][k]=F->Domain[k][0];
                if ( CH[i][k]>F->Domain[k][1] ) CH[i][k]=F->Domain[k][1];
            }
        }
        Force[i] = F->value ( CH[i] );
        if ( vilog_gati ) printf ( "%d\n", fitness_call );
        fitness_call++;
        if ( Force[i]>btg_value )
        {
            btg_value = Force[i];
            btg=CH[i];
            j=i;
        }
    }

    if ( btg_value > bsf_value )
    {
        memcpy ( bsf, CH[j], F->Dim*sizeof ( double ) );
        bsf_value = btg_value;
    }
}	

void gartou::SELECT ( void )
{
    double *h;
    int i1, i2, dead, last;

    while ( ActualSize > SelectedSize )
    {
        i1 = random_int ( 0,ActualSize-1 );
        i2 = random_int ( 1,ActualSize-1 );
        if ( i1==i2 ) i2--;
        if ( Force[i1] >= Force[i2] ) dead = i2;
        else dead = i1;
        last = ActualSize-1;
        h = CH[last];
        CH[last] = CH[dead];
        CH[dead] = h;
        Force[dead] = Force[last];
        ActualSize--;
    }
}	

void gartou::MUTATE ( void )
{
    double p, *x;
    int i, j, index;

    for ( i=0; i<SelectedSize; i++ )
    {
        if ( ActualSize == PoolSize ) break;
        p = random_double ( 0,1 );
        if ( p<=radioactivity )
        {
            index = random_int ( 0,SelectedSize-1 );
            x = new_point();
            for ( j=0;  j<F->Dim; j++ )
            {
                CH[ActualSize][j] = CH[index][j]+mutation_rate*( x[j]-CH[index][j] );
            }
            delete [] x;
            ActualSize++;
        }
    }
}

void gartou::LOCAL_MUTATE ( void )
{
    double p,dCH ;
    int i,j,index, sign ;
    for ( i=0; i<SelectedSize; i++ )
    {
        if ( ActualSize == PoolSize ) break;
        p = random_double( 0,1 );
        if ( p <= local_radioactivity )
        {
            index = random_int( 0, SelectedSize-1 );

            for ( j=0; j<F->Dim; j++ )
            {
                dCH = random_double( -mutagen[j],mutagen[j] );
                CH[ActualSize][j] = CH[index][j]+dCH;
            }
            ActualSize++;
        }
    }
}

void gartou::CROSS ( void )
{
    int i1,i2,i3,j ;
    while ( ActualSize < PoolSize )
    {
        i1 = random_int( 0,SelectedSize-1 );
        i2 = random_int( 1,SelectedSize-1 );
        if ( i1==i2 ) i2--;
        i3 = random_int( 0,SelectedSize-1 );
        for ( j=0 ; j<F->Dim ; j++ )
        {
            CH[ActualSize][j] = CH[i3][j]+cross_rate*( CH[i2][j]-CH[i1][j] ) ;
        }
        ActualSize++;
    }
}

void gartou::FIRST_GENERATION ( void)
{
    int i;

    Force = new double [PoolSize];
    CH = new p_double [PoolSize];
    for ( i=0; i<PoolSize; i++ )
    {
        CH[i] = new_point ();
    }
    bsf = new double [F->Dim];
    ActualSize = PoolSize;
    EVALUATE_GENERATION ( 0 );
    SELECT ();
}	

void gartou::clear_pool ( void )
{
    int i;
/*
    for ( i=0; i<PoolSize; i++ ) delete [] CH[i];
    delete [] CH;
    delete [] Force;
    delete [] bsf;
    delete [] mutagen;
*/
    for ( i=0; i<PoolSize; i++ ) if ( CH[i] ) delete [] CH[i];
    if ( CH ) delete [] CH;
    if ( Force ) delete [] Force;
    if ( bsf ) delete [] bsf;
    if ( mutagen ) delete [] mutagen;
}

// -------------------------------------- CERAF TECHNOLOGY -------------------------------------------------

void gartou::init_ceraf ( void )
{
    Extremes = new extreme [ExtremeSize];
}

int gartou::new_ceraf ( void )
{
    if ( btg_value-lastbtg > F->precision/2 )
    {
        lastchange=generation;
        lastbtg = btg_value;
    }

    if ( ( generation-lastchange ) < quiet_generation ) return 1;
    btg_value=MinDouble;
    lastbtg=MinDouble;
    lastchange=generation;
    Extremes[id].location = new double [F->Dim];
    Extremes[id].radius = new double [F->Dim];
    memcpy ( Extremes[id].location, btg, F->Dim*sizeof ( double ) );
    Extremes[id].value = btg_value;
    int i;
    for ( i=0; i<F->Dim; i++ )
    {
        Extremes[id].radius[i]=radius_rate*(F->Domain[i][1]-F->Domain[i][0]);
    }
    id++;
    n_extreme++;

    for ( i=0; i<SelectedSize; i++ ) CH[i]=new_point();

    return 0;
}

void gartou::check_ceraf_distance ( int start )
{
    int i, j, k;
    double r;

    for ( i=start*SelectedSize; i<PoolSize; i++ )
    {
        for ( j=0; j<n_extreme; j++ )
        {
            r=0.0;
            for ( k=0; k<F->Dim; k++ )
            {
                r += pow ( 2*( CH[i][k]-Extremes[j].location[k] )/Extremes[j].radius[k], 2 );
            }
            if ( r < 1 )
            {
                if ( i-SelectedSize > monsters )
                {
                    for ( k=0; k<F->Dim; k++ ) Extremes[j].radius[k]*=disinteg_rate;
                }
                delete [] CH[i];
                CH[i] = new_point ();
                j=n_extreme;
                i--;
                killed++;
            }
        }
    }
}

void gartou::clear_ceraf ( void )
{
    int i;
/*
    for ( i=0; i<n_extreme; i++ )
    {
        delete [] Extremes[i].location;
        delete [] Extremes[i].radius;
    }
    delete [] Extremes;
    delete [] all_locations;
    delete [] all_radius;
    */
    for ( i=0; i<n_extreme; i++ )
    {
        if ( Extremes[i].location ) delete [] Extremes[i].location;
        if ( Extremes[i].radius ) delete [] Extremes[i].radius;
    }
    if ( Extremes ) delete [] Extremes;
}


// -------------------------------------- STOPPING CRITERIA ------------------------------------------------

int gartou::to_continue ()
{
    if ( fitness_call > fitness_calls_limit ) return 0;
    if ( F->optimum ) if ( *F->optimum-bsf_value<=F->precision ) return 0;
    return 1;
}

// -------------------------------------- MAIN -------------------------------------------------------------

void gartou::run ( double *assessment=NULL )
{
    int stop=0, ceraf=1;
    initialize_randoms ();
    configuration ();
    init_vilog ();
    init_ceraf ();

    FIRST_GENERATION ();

    while ( !stop )
    {
        MUTATE ();
        monsters=ActualSize-SelectedSize;
        LOCAL_MUTATE ();
        CROSS ();
        check_ceraf_distance ( ceraf );
        EVALUATE_GENERATION ( ceraf );
        SELECT ();
        ceraf = new_ceraf ();
        if ( !vilog_news ( ceraf ) ) stop=1;
        if ( !to_continue () ) stop=1;
        generation++;
    }
    vilog_results ();
    vilog_make_out ();

    F->evaluate ( bsf );
    if ( assessment ) *assessment = bsf_value/(double)fitness_call;

    close_vilog ();
    clear_pool ();
    clear_ceraf ();
}