/*
 * ga.cpp		ver. 1.1
 *
 * Soubor s GA
 *
 * M. Leps, 7.7.99
 *
 */

#if !defined ( __GA__ )
#include "ga.h"
#endif

#include <stdio.h>
#include <time.h>
#include <math.h>

/**************************************************************************/

/************/
/* promenna */
/************/

/**************************************************************************/

void tprom::init_n ( void )
{
  switch ( t ) {
  case Cele :
    n=( int )ceil( log( ceil(( max.i-min.i )/p.i)+1 )/log( 2. )) ;
    break ;
  case Real :
    n=( int )ceil( log( ceil(( max.d-min.d )/p.d)+1 )/log( 2. )) ;
    break ;
  default :
    abort() ;
  }
}

bit tprom::get_int ( tuni on )
{
  bit tmp=0 ;

  switch ( t ) {
  case Cele :
    tmp=( bit )ceil(( on.i-min.i )/p.i) ;
	 break ;
  case Real :
    tmp=( bit )ceil(( on.d-min.d )/p.d) ;
	 break ;
  }
  return( tmp ) ;
}

tuni tprom::get_value ( bit on )
{
  tuni tmp ;

  switch ( t ) {
  case Cele :
    tmp.i=on*p.i+min.i ;
/*	 if( tmp.i>max.i )
		tmp.i=max.i ;*/
	 break ;
  case Real :
    tmp.d=on*p.d+min.d ;
/*	 if( tmp.d>max.d )
		tmp.d=max.d ;*/
	 break ;
  }
  return( tmp ) ;
}

tuni tprom::get_random ( void )
{
  tuni tmp ;

  switch ( t ) {
  case Cele :
    tmp.i=min.i+rnd_int( max.i-min.i ) ;
    break ;
  case Real :
    tmp.d=min.d+( max.d-min.d )*rnd_real() ;
    break ;
  }
  return( tmp ) ;
}

tuni tprom::get_max ( void )
{
  tuni tmp ;

  switch ( t ) {
  case Cele :
	 tmp.i=max.i ;
	 break ;
  case Real :
	 tmp.d=max.d ;
	 break ;
  }
  return( tmp ) ;
}
void tprom::print_tuni ( tuni ot )
{
  switch ( t ) {
  case Cele :
	 printf( " %10d", ot.i ) ;
	 break ;
  case Real :
	 printf( " %10f", ot.d ) ;
	 break ;
  }
}

/**************************************************************************/

/*******/
/* gen */
/*******/

/**************************************************************************/

tgen::tgen ( void )
{
  f=0. ;
  s=0. ;
  s1=0. ;
  c=0. ;
  bin=null ;
  pt=null ;
  pr=null ;
  pp=null ;
}

tgen::~tgen ( void )
{
  pr=null ;
  pp=null ;
  if ( pt ) delete [] pt ;
  if ( bin ) delete [] bin ;
}

void tgen::init ( int oid, int on, tprom *opr, tpop *op )
{
  id=oid ;
  bin=new bit[on] ;
  pt=new tuni[on] ;
  nb=on ;
  pr=opr ;
  pp=op ;

  for( int i=0 ; i<on ; i++ ) {
    bin[i]=0 ;
    pt[i].d=0.0 ;
    pt[i].i=0 ;
  }
}

void tgen::set_f ( void )
{
  switch ( pp->igl ) {
  case GlobalMin :
    s1=f ;
    s=1.0/( f+0.0000001 ) ;
    break ;
  case GlobalMax :
    s=f ;
    s1=1.0/( f+0.0000001 ) ;
    break ;
  }
}

void tgen::set_s ( void )
{
  const double smax=1.0 ;
  const double smin=0.05 ;

  switch ( pp->igl ) {
  case GlobalMin :
    s=( smax-smin )*( f-( pp->fmin ) )/( ( pp->fmin )-( pp->fmax)+0.0000001 )+smax ; /* f^ s_ */
    s1=( smax-smin )*( f-( pp->fmax ) )/( ( pp->fmax )-( pp->fmin )+0.0000001 )+smax ;
    break ;
  case GlobalMax :
    s=( smax-smin )*( f-( pp->fmax ) )/( ( pp->fmax )-( pp->fmin )+0.0000001 )+smax ; /* f^ s^ */
    s1=( smax-smin )*( f-( pp->fmin ) )/( ( pp->fmin )-( pp->fmax )+0.0000001 )+smax ;
    break ;
  }
}

void tgen::operate ( tgen &g2 ) 
{
  for( int i=0 ; i<nb ; i++ ) {
    if( pr[i].opt==Free ) {
      /*this.*/cross( g2.bin, i ) ;
      if( rnd_real()<Pmutat )
	/*this.*/mutace( i ) ;
      if( rnd_real()<Pmutat )
	g2.mutace( i ) ;
    }
  }
  /*this.*/update_res() ;
  g2.update_res() ;
}

void tgen::mutace ( int oi )
{
  int kde ;
  bit m, n ;

  kde=rnd_int( pr[oi].n ) ;
  m=1 ;
  m<<=kde ;
  n=bin[oi] ;
  bin[oi]=(( ~m ) & n ) | ( m & ( ~n )) ;
}

void tgen::cross ( bit *bbin, int oi )
{
  int kde ;
  bit o, p, m, n ;

  kde=rnd_int( pr[oi].n+1 ) ;
  o=bin[oi] ;
  p=bbin[oi] ;
  m=~0x0 ;
  m<<=kde ;
  n=~m ;
  bin[oi]=( o&m ) | ( p&n ) ;
  bbin[oi]=( o&n ) | ( p&m ) ;

}
 
void tgen::grandom ( void )
{
  for ( int i=0 ; i<nb ; i++ ) {
    if ( pr[i].opt==Free ) {
      pt[i]=pr[i].get_random() ;
      bin[i]=pr[i].get_int( pt[i] ) ;
    }
  }
}

void tgen::set_max( void ) 
{
  for ( int i=0 ; i<nb ; i++ ) {
    if ( pr[i].opt==Free ) {
		pt[i]=pr[i].get_max() ;
      bin[i]=pr[i].get_int( pt[i] ) ;
    }
  }
}
		

void tgen::update_bin ( void )
{
  for( int i=0 ; i<nb ; i++ )
    bin[i] = pr[i].get_int( pt[i] ) ;
}

void tgen::update_res ( void )
{
  for( int i=0 ; i<nb ; i++ )
    pt[i] = pr[i].get_value( bin[i] ) ;
}

void tgen::print_res ( void )
{
  printf( "%6d", id-1 ) ;
  for( int i=0 ; i<nb ; i++ ) 
    pr[i].print_tuni( pt[i] ) ;
  printf( " c%10f f%10f s1%10f s%10f\n", c, f, s1, s ) ;
}

/**************************************************************************/

/************/
/* populace */
/************/

/**************************************************************************/

tpop::tpop ( void )
{
  time_t t ;
  FILE *f ;

  t = time( &t ) ;
  srand48( t ) ;
  srand( t ) ;

  pr=null ;
  pg=null ;
  psc=null ;
  ps=null ;

  f=fopen( FileAvg, Write ) ;
  fclose( f ) ;
  f=fopen( FileBst, Write ) ;
  fclose( f ) ;
  f=fopen( FileWrs, Write ) ;
  fclose( f ) ;
  f=fopen( FilePrb, Write ) ;
  fclose( f ) ;

  /* pro testovani, nutno do konecne verze upravit ! */
  select=Rsis ;

  /*  igl=GlobalMin ; its=PowerS ;*/
}

tpop::~tpop ( void )
{
  if( psc ) delete psc ;
  if( ps ) delete ps ;
  if( pg ) delete [] pg ;
  if( pr ) delete [] pr ;
}

void tpop::init_mem ( int ong )
{
  switch ( imt ) {
  case Bett :
    ngnew=2 ; break ;
  case Gold :
    ngnew=ong ; break ;
  }

  ngall=ong+ngnew ;       /* plus 1 pro nejlepsiho */
  pg=new tgen[ ngall+2 ] ;    /* plus geny na novou populaci */

  for( int i=0 ; i<ngall+2 ; i++ )
    pg[i].init( i+1, npr, pr, this ) ;

  pg[ ngall ].set_max() ;

  
  psc=new tscale( ong, pg ) ;

  /* alokace vyberu */
  switch ( select ) {
  case General :
    ps=new tselect( ong, 10, pg ) ; break ;
  case Ruleta :
    ps=new truleta( ong, 10, pg ) ; break ;
  case Rsis :
    ps=new trsis( ong, 10, pg ) ; break ;
  default :
    abort() ;
  }

  ng=ong ;
  it=0 ;
  itt=0 ;
  pru=0. ;

  fmax=1.0 ;
  fmin=0.0 ;
}

void tpop::init_sum ( void )
{
  double sumF=0. ;

  sumS=0. ;
  sumS1=0. ;
  for( int i=0 ; i<ng ; i++ ) {
    sumF+=pg[i].f ;
    sumS+=pg[i].s ;
    sumS1+=pg[i].s1 ;
  }
  pru=sumF/ng ;
}

void tpop::update_sum ( int ogo, int ogn )
{
  sumS-=pg[ogo].s ;
  sumS1-=pg[ogo].s1 ;
  sumS+=pg[ogn].s ;
  sumS1+=pg[ogn].s1 ;
  pru=sumS/ng ;
}

void tpop::set_F ( void ( *fce )( tuni *p, double &f, double &c ))
{
  pfce=fce ;
}

void tpop::get_F ( int og )
{
  pfce( pg[og].pt, pg[og].f, pg[og].c ) ;
}

void tpop::run2 ( int on, int ot )
{
  int j ;
  int count=0 ;

  print_avg_f() ;

  for( int i=0 ; i<on ; i++ ) {
    it++ ;
    count++ ;
    for( j=0 ; j<ng ; j+=2 ) {
      itt+=2 ;
      copy_bin( ng+j , ruleta( MaxS ) ) ;
      copy_bin( ng+j+1 , ruleta( MaxS ) ) ;
      pg[ng+j].operate( pg[ng+j+1] ) ;
      /*this.*/get_F( ng+j ) ;
      /*this.*/get_F( ng+j+1 ) ;
    }
    for( j=0 ; j<ng ; j++ )
      copy_gen( j , j+ng ) ;

    if( its==PowerS ) {
      fmax=pg[find_best( GlobalMax )].f ;
      fmin=pg[find_best( GlobalMin )].f ;
      
      for( j=0 ; j<ng ; j++ ) {
		  pg[j].set_s() ;
      }
    }
    else { /* PowerF */
      for( j=0 ; j<ng ; j++ ) {
		  pg[j].set_f() ;
      }
    }
    init_sum() ;
    print_avg_f() ;
    /*    print_all () ;*/
    /*print_best() ;
    print_stat() ;*/

    if( count>=ot ) {
      /*  print_all () ;*/
      print_best() ;
      /*print_stat() ;*/
	   
      count=0 ;
    }
  }
}

void tpop::run3 ( int on, int ot )
{
  int j ;
  int count=0 ;

  print_avg_f() ;
  
  /*this.*/get_F( ngall) ;

  for( int i=0 ; i<on ; i++ ) {
    it++ ;
    count++ ;

    /* vyber !!! */
    ps->set_id() ;

    /* ps->print_selected() ; */

    for( j=0 ; j<ng ; j+=2 ) {
      itt+=2 ;
      /*#define RULETA*/
#ifndef RULETA
      copy_bin( ng+j , ps->get_id() ) ;
      copy_bin( ng+j+1 , ps->get_id() ) ;
#else
      copy_bin( ng+j , ruleta( MaxS ) ) ;
      copy_bin( ng+j+1 , ruleta( MaxS ) ) ;
#endif

      pg[ng+j].operate( pg[ng+j+1] ) ;
      /*this.*/get_F( ng+j ) ;
      /*this.*/get_F( ng+j+1 ) ;
    }

    for( j=0 ; j<ng ; j++ )
      copy_gen( j , j+ng ) ;

    switch( its ) {
    case PowerS :
      fmax=pg[find_best( GlobalMax )].f ;
      fmin=pg[find_best( GlobalMin )].f ;
      
      for( j=0 ; j<ng ; j++ ) {
	pg[j].set_s() ;
      }
      break ;
    case PowerF :
      for( j=0 ; j<ng ; j++ ) {
	pg[j].set_f() ;
	/*printf( "pokus\n" ) ;*/
      }
      break ;
    case GoldF :
      for( j=0 ; j<ng ; j++ )    /* Normal prevod */
	pg[j].set_f() ;
      psc-> get_scaled( MaxS ) ; /* Goldberg */
      break ;
    default :
      abort() ;
    }

	 /* nejlepsi do ngall+1 */
	 int ibst ;
	 ibst=find_best( GlobalMin ) ;
	 if( pg[ibst].f<pg[ngall].f ) {
		copy_gen( ngall , ibst ) ;
  		printf( "** \n" ) ;
/*		printf( "BEST \n\n\n" ) ;*/
	 }

#ifdef NEJHORSI
	 ibst=find_best( GlobalMax ) ;
	 if( pg[ibst].f>pg[ngall+1].f ) {
		copy_gen( ngall+1 , ibst ) ;
  		printf( "-- \n" ) ;
	 }
#endif

    init_sum() ;
    print_avg_f() ;
    /*    print_all () ;*/
    /* print_best() ; */

    if( count>=ot ) {
      /*  print_all () ;*/
      print_best() ;
      /*print_stat() ;*/
	   
      count=0 ;
    }
  }
}

void tpop::run ( int on, int ot )
{
  int j=0 ;
  int count=0 ;

  print_avg_f() ;

  for( int i=0 ; i<on ; i++ ) {

    /*this.*/sga_in() ;
    /*this.*/sga_out() ;
    
    it++ ;
    itt+=2 ;
	 count+=2 ;
    j++ ;

    if( count>=50 ) {
		print_avg_f() ;
      count=0 ;
	 }	 

    if( j>=ot ) {
      /*  print_all () ;*/
      print_best() ;
      /*print_stat() ;*/
	   
      j=0 ;
    }
  }
}

void tpop::random_gen ( void )
{
  int i ;
  for( i=0 ; i<ng ; i++ ) {
    pg[i].grandom() ;
    /*this.*/get_F( i ) ;
  }

  switch ( its ) {
  case PowerF :
    for( i=0 ; i<ng ; i++ ) {
      pg[i].set_f() ;
      /*pg[i].print_res() ;*/
    }
    break ;
  case PowerS :
    fmax=pg[find_best( GlobalMax )].f ;
    fmin=pg[find_best( GlobalMin )].f ;
    
    for( i=0 ; i<ng ; i++ ) {
      pg[i].set_s() ;
      /*pg[i].print_res() ;*/
    }
    break ;
  case GoldF :
    for( i=0 ; i<ng ; i++ ) 
      pg[i].set_f() ;
    psc-> get_scaled( MaxS ) ;
    break ;
  default :
	 abort() ;
  }

  /*this.*/init_sum() ;
}

void tpop::copy_gen ( int oi1, int oi2 )
{
  pg[oi1].f = pg[oi2].f ;
  pg[oi1].s = pg[oi2].s ;
  pg[oi1].s1 = pg[oi2].s1 ;
  pg[oi1].c = pg[oi2].c ;
  memcpy( pg[oi1].bin, pg[oi2].bin, npr*sizeof( bit ) ) ;
  memcpy( pg[oi1].pt, pg[oi2].pt, npr*sizeof( tuni ) ) ;
}

void tpop::copy_bin ( int oi1, int oi2 )
{
  memcpy( pg[oi1].bin, pg[oi2].bin, npr*sizeof( bit ) ) ;
}

int tpop::find_best ( int ot )
{
  int i, kde=0 ;

  if( ot==GlobalMin ) {
    for( i=1 ; i<ng ; i++ ) {
      if( pg[i].f < pg[kde].f )
		  kde=i ;
    }
  }
  else {
    for( i=1 ; i<ng ; i++ ) {
      if( pg[i].f > pg[kde].f )
	kde=i ;
    }
  }
  return( kde ) ;
}

int tpop::ruleta ( int oc )
{
  int i ;
  double interval=0. ;
  double kde ;

  kde=rnd_real() ;
  if( oc==MinS ) { /* vetsi pravdepodobnost pro mensi s */
    for( i=0 ; i<ng ; i++ ) {
      interval+=pg[i].s1/sumS1 ;
      if( interval>kde )
	break;
    }
  }
  else { /* vetsi pravdepodobnost pro vetsi s */
    for( i=0 ; i<ng ; i++ ) {
      interval+=pg[i].s/sumS ;
      if( interval>kde )
	break;
    }
  }
  return( i ) ;
}

void tpop::sga_in ( void )
{
  int i, t1, t2 ;
  
  t1=ruleta( MaxS ) ;  copy_bin( ng, t1 ) ;
  t2=ruleta( MaxS ) ;  copy_bin( ng+1, t2 ) ;

  /*  this.print_old( t1, t2 ) ; */
  /*  copy_gen( ng+1, ruleta( Kladna ) ) ;*/
  
  /* pro vsechny prom bez omezeni */
  pg[ng].operate( pg[ng+1] ) ;

  /*this.*/get_F( ng ) ;
  /*this.*/get_F( ng+1 ) ;

  switch ( its ) {
  case PowerF :
    pg[ng].set_f() ;
    pg[ng+1].set_f() ;
    break ;
/*  case PowerS :*/
  default :
    fmax=pg[find_best( GlobalMax )].f ;
    fmin=pg[find_best( GlobalMin )].f ;
    
    for( i=0 ; i<ng+2 ; i++ ) {
      pg[i].set_s() ;
    }
    break ;
  }
  /*this.print_new() ;*/
}

void tpop::sga_out ( void )
{
  int kde ;

  switch ( its ) {
  case PowerF :
    kde=ruleta( MinS ) ;
    if( pg[ng].s > pg[kde].s ) {
      //update_sum( kde, ng ) ;
      copy_gen( kde, ng ) ;
    }
    kde=ruleta( MinS ) ;
    if( pg[ng+1].s > pg[kde].s ) {
      //update_sum( kde, ng+1 ) ;
      copy_gen( kde, ng+1 ) ;
    }
    init_sum() ;
    break ;
  case PowerS :
    int is=0 ;
    kde=ruleta( MinS ) ;
    if( pg[ng].s > pg[kde].s ) {
      is=1 ;
      copy_gen( kde, ng ) ;
    }
    kde=ruleta( MinS ) ;
    if( pg[ng+1].s > pg[kde].s ) {
      is=1 ;
      copy_gen( kde, ng+1 ) ;
    }
    if( is!=0 )
      init_sum() ;
    break ;
  }
}

void tpop::load_pr ( char *jm )
{
  int i, c ;
  FILE *f ;

  if (( f=fopen( jm, Read ))==NULL ) {
	 printf( "Soubor %s se nepodarilo otevrit \n", jm ) ;
	 abort() ;
  }

  while(( c=getc( f ))!='\n' ) ;
  fscanf( f, "%d \n", &npr ) ;
  while(( c=getc( f ))!='\n' ) ;
  i=npr ;
  pr=new tprom[npr];

  for( i=0 ; i<npr ; i++ ) {
    fscanf( f, "%d %c", &(pr[i].id), &(pr[i].t) ) ;
    if( pr[i].t==Cele ) {
      fscanf( f, "%d %d %d %d",\
	      &(pr[i].min.i), &(pr[i].max.i), &(pr[i].p.i), &(pr[i].opt) ) ;
    }
    else {
      fscanf( f, "%lf %lf %lf %d",\
	      &(pr[i].min.d), &(pr[i].max.d), &(pr[i].p.d), &(pr[i].opt) ) ;
    }
    pr[i].init_n() ;
  }

  if( fclose( f )==EOF ) {
	 printf( "Soubor %s se nepodarilo uzavrit \n", jm ) ;
	 abort() ;
  }
}

void tpop::print_best ( void )
{
  printf( "best: %7d ", it ) ;
  pg[find_best( GlobalMin )].print_res() ;
}

void tpop::print_best_1 ( void )
{
  printf( "THE best ONE : %7d ", it ) ;
  pg[ngall].print_res() ;
}

void tpop::print_all ( void )
{
  int i ;

  printf( "all:\n" ) ;
  for( i=0 ; i<ng ; i++ )
	 pg[i].print_res() ;
  /*this.*/print_stat() ;
}

void tpop::print_old ( int o1, int o2 )
{
  printf( "old:\n" ) ;
  pg[o1].print_res() ;
  pg[o2].print_res() ;
}
 
void tpop::print_new ( void )
{
  printf( "new:\n" ) ;
  pg[ng].print_res() ;
  pg[ng+1].print_res() ;
}

void tpop::print_stat ( void )
{
  printf( "SumS1: %10f SumS: %10f PruF: %10f \n", sumS1, sumS, pru ) ;
}

void tpop::print_avg_f ( void ) 
{
  FILE *f ;
  if (( f=fopen( FileAvg, Append ))==NULL ) {
	 printf( "Soubor graf se nepodarilo otevrit \n" ) ;
	 abort() ;
  }

  fprintf( f, " %5d %11.6f \n", itt, pru ) ;

  fclose( f ) ;

  if (( f=fopen( FileBst, Append ))==NULL ) {
	 printf( "Soubor graf se nepodarilo otevrit \n" ) ;
	 abort() ;
  }

  fprintf( f, " %5d %11.6f \n", itt, pg[find_best( GlobalMin )].f ) ;

  fclose( f ) ;

  f=fopen( FileWrs, Append ) ;
  fprintf( f, " %5d %11.6f \n", itt, pg[find_best( GlobalMax )].f ) ;
  fclose( f ) ;
}

#ifdef __BH__
void tpop::print_fce ( void ) 
{
  double bb, bh, fb=2000. ;
  FILE *f ;
  f=fopen( "data/funkce.grf", Write ) ;
  for( double b=0.15 ; b<0.45 ; b+=0.005 ) {
	 for( double h=0.15 ; h<0.65 ; h+=0.005 ) {
		pg[0].pt[0].d=b ;
		pg[0].pt[1].d=h ;
		pg[0].pt[2].i=5 ;
		pg[0].pt[3].i=5 ;
		pg[0].pt[4].i=3 ;
		pg[0].pt[5].i=3 ;
		pg[0].pt[6].i=2 ;
		pg[0].pt[7].i=0 ;
		pg[0].pt[8].i=0 ;
		pg[0].pt[9].i=6 ;
		pg[0].pt[10].i=4 ;
		pg[0].pt[11].i=4 ;
		
		get_F( 0 ) ;

		if( fb>pg[0].f ) {
		  fb=pg[0].f ;
		  bb=b ;
		  bh=h ;
		}

		fprintf( f, "%5.3f\t%5.3f\t%10.5f\n", b, h, pg[0].f ) ;
	 }
	 fprintf( f, "\n") ;
	 printf( "*\n" ) ;
  }
  printf( "Best F : %5.3f\t%5.3f\t%10.5f\n", bb, bh, fb ) ; 
  fclose( f ) ;
}
#endif

void tpop::print_fce ( void ) 
{
  double fb=2000. ;
  int b1 ;
  double b2 ;
  FILE *f ;
  f=fopen( "data/funkce.grf", Write ) ;
  for( int i=0 ; i<16 ; i++ ) {
	for( double h=0.15 ; h<0.65 ; h+=0.025 ) {
		pg[0].pt[0].d=0.15 ;
		pg[0].pt[1].d=h ;
		pg[0].pt[2].i=i ;
		pg[0].pt[3].i=5 ;
		pg[0].pt[4].i=3 ;
		pg[0].pt[5].i=3 ;
		pg[0].pt[6].i=2 ;
		pg[0].pt[7].i=0 ;
		pg[0].pt[8].i=0 ;
		pg[0].pt[9].i=6 ;
		pg[0].pt[10].i=4 ;
		pg[0].pt[11].i=4 ;
		
		get_F( 0 ) ;

		if( fb>pg[0].f ) {
		  fb=pg[0].f ;
		  b1=i ;
		  b2=h ;
		}

		fprintf( f, "%3d\t%5.3f\t%10.5f\n", i, h, pg[0].f ) ;
	 }
	 fprintf( f, "\n") ;
	 printf( "*\n" ) ;
  }
  printf( "Best F : %3d\t%5.3f\t%10.5f\n", b1, b2, fb ) ; 
  fclose( f ) ;
}
