/*
 * ga.h		ver. 1.1
 *
 * Hlavickovy soubor metody GA
 *
 * M. Leps, 7.7.1999
 *
 */

#ifndef __GA__
#define __GA__

#include "defs.h"

#define FileAvg "data/graf_avg.grf"
#define FileBst "data/graf_bst.grf"
#define FileWrs "data/graf_wrs.grf"

#define FilePrb "data/data_prb.dat"

/* typ pro zapis binarniho cisla */
typedef unsigned long int bit ;

/* obecny typ */
typedef union
{
  int i ;
  double d ;
} tuni ;

/* typ promenne */
struct tprom
{
  int id ;                                                      /* cislo promenne */
  int n ;                                                       /* pocet platnych bitu */
  tuni max, min ;                                               /* maximum, minimum */
  tuni p ;                                                      /* presnost */
  int opt ;                                                     /* priznak optimalizace */
  char t ;                                                      /* typ promenne */

  tprom ( void ) { opt=Lock ; } ;
 ~tprom ( void ) {} ;

  void init_n ( void ) ;
  bit  get_int ( tuni on ) ;
  tuni get_value ( bit on ) ;
  tuni get_random ( void ) ;
  tuni get_max ( void ) ;
  void print_tuni ( tuni ot ) ;
} ;

struct tpop ;

struct tgen
{
  int nb ;                                                      /* pocet promennych */
  int id ;                                                      /* cislo genu */
  double f ;                                                    /* funkcni hodnota */
  double s ;                                                    /* sila chromozomu */
  double s1 ;                                                   /* inverzni sila chromozomu */
  double c ;                                                    /* cena, dalsi sledovany parametr */
  

  bit *bin ;                                                    /* binarni pole */
  tuni *pt ;                                                    /* ukazatel na reseni */
  tprom *pr ;                                                   /* ukazatel na promenne */
  tpop *pp ;                                                    /* pt. na populaci */

  tgen ( void ) ;
 ~tgen ( void ) ;

  void init ( int oid, int on, tprom *opr, tpop *op ) ;

  void set_f ( void ) ;
  void set_s ( void ) ;

  void operate ( tgen &g2 ) ;

  void mutace ( int oi ) ;
  void cross ( bit *bbin, int oi ) ;

  void grandom ( void ) ;
  void set_max ( void ) ;
  void update_bin ( void ) ;
  void update_res ( void ) ;
  void print_res ( void ) ;
} ;

#include "scale.h"
#include "select.h"

struct tpop {
  int npr ;                                                     /* pocet promennych */
  int ng ;                                                      /* pocet chromozomu */
  int ngnew ;                                                   /* # genu v nove populaci */
  int ngall ;                                                   /* # vsech genu */
  int it ;                                                      /* pocet iter = stari */
  int itt ;
  int igl ;                                                     /* typ metody min/max */
  int its ;                                                     /* typ s/f sily genu */
  int imt ;                                                     /* metoda Goldberg/Leps */
  double pru ;                                                  /* prumer sily genu */
  double fmax, fmin ;                                           /* max, min optima fce */
  double sumS ;                                                 /* soucet sil */
  double sumS1 ;                                                /* soucet prevracenych sil */
  tprom *pr ;                                                   /* ukazatel na vl. prom. */
  tgen *pg ;                                                    /* ukazatel na chrom. */
  SCALE  scale ;
  tscale *psc ;                                                 /* typ scalingu */
  SELECT select ;                                               /* typ vyberu */
  tselect *ps ;                                                 /* pt. na vyber */
  void ( *pfce )( tuni *p, double &f, double &c) ;

  tpop ( void ) ;
 ~tpop ( void ) ;

  void init_mem ( int ong ) ;
  void init_sum ( void ) ;
  void update_sum ( int ogo, int ogn ) ;

  void set_F ( void ( *fce )( tuni *p, double &f, double &c )) ;
  void get_F ( int og ) ;

  void run ( int on, int ot ) ;         /* on-the-fly */
  void run2 ( int on, int ot ) ;        /* SGA ruleta */
  void run3 ( int on, int ot ) ;        /* SGA RSMWR */

  void random_gen ( void ) ;  /* init */

  void copy_gen ( int oi1, int oi2 ) ;
  void copy_bin ( int oi1, int oi2 ) ;

  int find_best ( int ot ) ;
  int ruleta ( int oc ) ;
  void sga_in ( void ) ;
  void sga_out ( void ) ;

  void load_pr ( char *jm ) ;
  void print_best ( void ) ;
  void print_best_1 ( void ) ;
  void print_all ( void ) ;
  void print_old ( int o1, int o2 ) ;
  void print_new ( void ) ;
  void print_stat ( void ) ;
  void print_avg_f ( void ) ;
  void print_fce ( void ) ;
} ;

#endif
