#include "mr.beam.h"
#include "misc.h"

mr_beam::mr_beam ( void ) : objective_function ()
{
    int i ;

    Dim=18 ;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ ) Domain[i] = new double [3];

    //b
    for ( i=0; i<1; i++ )
    {
        Domain[i][0]=0.15;
        Domain[i][1]=0.45;
        Domain[i][2]=0.025;
    }
    //h
    for ( i=1; i<2; i++ )
    {
        Domain[i][0]=0.15;
        Domain[i][1]=0.85;
        Domain[i][2]=0.025;
    }
    //horni, dolni pr
    for ( i=2; i<4; i++ )
    {
        Domain[i][0]=0.0;
        Domain[i][1]=9.0;
        Domain[i][2]=1.0;
    }
    // N
    for ( i=4; i<10; i++ )
    {
        Domain[i][0]=0.0;
        Domain[i][1]=12.0;
        Domain[i][2]=1.0;
    }
    // Li pro M
    for ( i=10; i<12; i++ )
    {
        Domain[i][0]=0.0;
        Domain[i][1]=12.0;
        Domain[i][2]=1.0;
    }
    //Prw
    for ( i=14; i<15; i++ )
    {
        Domain[i][0]=0.0;
        Domain[i][1]=3.0;
        Domain[i][2]=1.0;
    }
    //Li pro W
    for ( i=12; i<14; i++ )
    {
        Domain[i][0]=0.0;
        Domain[i][1]=12.0;
        Domain[i][2]=1.0;
    }
    //SW
    for ( i=15; i<18; i++ )
    {
        Domain[i][0]=0.05;
        Domain[i][1]=0.4;
        Domain[i][2]=0.025;
    }
    optimum = new double;
    *optimum=1.748;
    precision=0.001;
    sprintf ( name, "mr.beam" );
    outfile = new char [256];
    outfile = &(name[0]);
    Return_to_domain=1;

    v=new double[Dim] ;
    pr.prepare( 21 ) ;
    TDimBeam *beam;
    beam=alokaceTDB(1) ;

    kce=TSKload( "mr.beam/test4.in" ) ;
    dimenzace( kce ) ;
}

mr_beam::~mr_beam ( void )
{
    delete [] v ;
//    delete kce ;
}

int mr_beam::new_case ( void )
{
    return 0 ;
}

int mr_beam::check_domain ( void )
{
    return 1 ;
}

int mr_beam::make_defined ( double *ox )
{
    double *&Chromo=ox ;
    int b=0,j;
    for ( j=0; j<Dim; j++ )
    {
        if ( Chromo[j] < Domain[j][0] )
        {
            Chromo[j]=random_double(Domain[j][0], Domain[j][1]);
            b++ ;
        }
        if ( Chromo[j] > Domain[j][1] )
        {
            Chromo[j]=random_double(Domain[j][0], Domain[j][1]);
            b++ ;
        }
    }
}

void mr_beam::new_point ( double *ox )
{
    int i ;
    for ( i=0 ; i<Dim ; i++ )
    {
        ox[i]=random_double(Domain[i][0], Domain[i][1]);
    }
}

void mr_beam::get_limits ( int odir , double &omin , double &omax )
{
    omin=Domain[odir][0] ;
    omax=Domain[odir][1] ;
}

void mr_beam::get_limits ( int odir , double &omin , double &omax , double &oprec )
{
    omin=Domain[odir][0] ;
    omax=Domain[odir][1] ;
    oprec=Domain[odir][2] ;
}

void mr_beam::get_value_range ( double &omin , double &omax )
{
    omin=0.0 ;
    omax=2.0 ;
}

double mr_beam::value ( double *ox )
{
    int j ;

    for ( j=0; j<Dim; j++ )
    {
        if ( ox[j] < Domain[j][0] ) return 0.0 ;
        if ( ox[j] > Domain[j][1] ) return 0.0 ;
    }
    round_chrm( ox,v ) ;

    pr.clear_res() ;

    pr[0].r = v[0] ; // pb->b;
    pr[1].r = v[1] ; //  pb->h;
    pr[2].i = (int) v[2] ; //  pb->pr[DOLNI];
    pr[3].i = (int) v[3] ; //  pb->pr[HORNI];
    pr[4].i = (int) v[4] ; //  pb->m[0].as[DOLNI].n;
    pr[5].i = (int) v[5] ; //  pb->m[1].as[DOLNI].n;
    pr[6].i = (int) v[6] ; //  pb->m[2].as[DOLNI].n;
    pr[7].i = (int) v[7] ; //  pb->m[0].as[HORNI].n;
    pr[8].i = (int) v[8] ; //  pb->m[1].as[HORNI].n;
    pr[9].i = (int) v[9] ; //  pb->m[2].as[HORNI].n;
    pr[10].i = (int) v[10] ; //  pb->m[0].s;
    pr[11].i = (int) v[11] ; //  pb->m[1].s;

    //smyk
    pr[12].i = (int) v[12] ; //  pb->w[0].s;
    pr[13].i = (int) v[13] ; //  pb->w[1].s;
    pr[14].i = (int) v[14] ; //  pb->prw;
    pr[15].r = v[15] ; //  pb->w[0].sw;
    pr[16].r = v[16] ; //  pb->w[1].sw;
    pr[17].r = v[17] ; //  pb->w[2].sw;
    pr[18].i = 2 ; //  pb->w[0].pw;
    pr[19].i = 2 ; //  pb->w[1].pw;
    pr[20].i = 2; //  pb->w[2].pw;

    OP_silaGen(  pr, kce->pdb ) ;

    return( 1.0/pr.f ) ;
}

int mr_beam::do_you_know_yourself ( void )
{
    return 1 ;
}

int mr_beam::to_be_stopped ( double obsf )
{
    if ( obsf>=1.748 ) return 1 ;
    return 0 ;
}

int mr_beam::how_many_extremes ( void )
{
    return 1 ;
}

double mr_beam::get_extreme ( int oi , double *ox )
{
    if ( oi!=0 ) return NoDouble ;
    if ( ox )
    {
        ox[0]=0.150 ;
        ox[1]=0.525 ;
        ox[2]=1.0 ;
        ox[3]=1.0 ;
        ox[4]=4.0 ;
        ox[5]=2.0 ;
        ox[6]=0.0 ;
        ox[7]=0.0 ;
        ox[8]=1.0 ;
        ox[9]=8.0 ;
        ox[10]=8.0 ;
        ox[11]=2.0 ;
        ox[12]=9.0 ;
        ox[13]=9.0 ;
        ox[14]=0.0 ;
        ox[15]=0.375 ;
        ox[16]=0.075 ;
        ox[17]=0.175 ;
    }
    return 1.748 ;
}

double mr_beam::get_global_extreme ( double *ox )
{
    if ( ox )
    {
        ox[0]=0.150 ;
        ox[1]=0.525 ;
        ox[2]=1.0 ;
        ox[3]=1.0 ;
        ox[4]=4.0 ;
        ox[5]=2.0 ;
        ox[6]=0.0 ;
        ox[7]=0.0 ;
        ox[8]=1.0 ;
        ox[9]=8.0 ;
        ox[10]=8.0 ;
        ox[11]=2.0 ;
        ox[12]=9.0 ;
        ox[13]=9.0 ;
        ox[14]=0.0 ;
        ox[15]=0.375 ;
        ox[16]=0.075 ;
        ox[17]=0.175 ;
    }
    return 1.748 ;
}

void mr_beam::round_chrm ( double *Chromo, double *Rounded )
{
    int i;

    for ( i=0; i<Dim; i++ )
    {
        Rounded[i] = floor ( Chromo[i]/Domain[i][2]+0.5 )*Domain[i][2];
    }
}

