/*
* rc.h		ver. 1.0
*
* Hlavickovy soubor pro projekt RC
*
* M. Leps, 3.12.1998
*
*/

#ifndef _RC_
#define _RC_

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <string.h>
#include <stdlib.h>

#include "adr.h"
#include "variable.h"

/* obecne definice pro funkce */

#define GLOBAL	/*  */
#define LOCAL	static

/* definice konstant */
#define MPI           3.14159

#define BAD			     0		   		/* hodnota pro zapornou odpoved */
#define GOOD	       1			   	/* hodnota pro kladnou odpoved */
#define ZMENA       -1          /* hodnota pro zmenu */

#define ERRLOWC		  -3				  /* nedostatek betonu (maly prurez) */
#define ERRLOWS		  -4				  /* nedostatek oceli (velky prurez) */
#define ERRLOWCEC  	-5				  /* nedostatek betonu dle EC (maly prurez) */

#define HORNI         1             /* tazena horni vlakna */
#define DOLNI         0             /* tazena dolni vlakna */

#define SMRAZEM       1             /* trida prostredi */
#define BEZMRAZU      0             /* trida prostredi */

#define NEWPOP        0             /* cena pro celou populaci */
#define NEWGEN        1             /* cena pro 2 nove geny */

#define PROM          2             /* pocet sledovanych prommenych */
#define PR_X          0
#define PR_KSI        1

/* definice materialovych konstatnt */
/*#define GA    1.15*/
/*#define GC    1.5*/
/*#define GCW   1.3*/

/********************************************************/
/* definice maker */
/********************************************************/

/********************************************************/
/* matematicke operace */

#ifndef RAND_MAX
#define RAND_MAX             32768
#endif


#define rnd(a) ((rand()>>3)%(a))
#define srnd() ((double)rand()/(double)(RAND_MAX))
#define sround(a) (floor((a)+0.5))

/********************************************************/
/* maticove operace pro matice ulozene v radcich */
#define mat(a, i, j, m) (a[(m)*(i)+(j)])
#define nulmat(i, n, a) \
	do { \
	for (i=0; i<n; i++) a[i] = 0; \
	} while(0)
#define plusmat(a, b, i, n) \
	do { \
	for (i=0; i<n; i++) a[i] += b[i]; \
	} while(0)

/********************************************************/
/* pristup k prvkum lokalni mt */
#define mtt(a, i, j) (mat(a,i,j,6))

/********************************************************/
/* Prepocte pocet prutu daneho typu na As */
#define npr_as(n, pr) (0.25*(n)*MPI*(pr)*(pr))
/*#define npr_g( n,i ) ( n*G_plocha[i] )*/
/*#define npr_w( n,i ) ( n*W_plocha[i] )*/
/* Vraci prumer z G_profil */
#define i_pr(i) (G_profil[i])
/* Vraci prumer z W_profil */
#define w_pr(i) (W_profil[i])

/********************************************************/
/* Aktualizace pointeru na akt.gen */
#define seta(p,por) ((p)->a = (p)->pg+(por))

/********************************************************/
/* definice globalnich typu */
/********************************************************/

/********************************************************/
/* prostredi */
struct TProstr {
  char fin[50];        /* vstupni soubor */
  char fopt[50];       /* optimalizacni soubor */
  int pocop;           /* pocet opakovani */
  int iter;            /* pocet iteraci */
  int krok;            /* pocet kroku */
  int pocg;            /* pocetgenu */
  int vyber;           /* vyber pocatku */
  int reseni;          /* typ reseni */
#ifndef DEBUG
  char out[50];        /* vystupni soubor */
#endif
};
typedef struct TProstr TProstr;

/********************************************************/
/* typ pro optimalizacni hodnoty */
/* hodnotici vl. */
struct TSPrice {
  double vc;           /* objem betonu */
  double vs;           /* objem vyztuze */
  double vsw;          /* objem trminku */
  double wc;           /* hmotnost betonu */
  double ws;           /* hmotnost oceli */
  double cc;           /* cena betonu */
  double cs;           /* cena oceli */
  double c;            /* celkova cena */
};
typedef struct TSPrice TSPrice;

/* bitova prommena */
typedef unsigned long int bit;

/* chromozom */
struct TSGen {
  int id;              /* cislo genu */
  double f;            /* sila chromozomu */
  double f1;           /* prevracena sila chromozomu */
  double c;            /* funkcni hodnota */
  bit *bin;            /* binarni pole */
};
typedef struct TSGen TSGen;

/* typ promenne */
typedef enum { I='I', D='D'} Typ;
/* navrhove promenne */
struct TSVect {
  int id;              /* cislo promenne */
  int n;               /* pocet platnych bitu */
  double f;            /* sila promenne */
  int opt;             /* priznak optimalizace */
  Typ t;               /* typ promenne */
};
typedef struct TSVect TSVect;

/* populace */
struct TSPop {
  int npr;             /* pocet promennych */
  int ngen;            /* pocet chromozomu */
  int it;              /* pocet iter = stari */
  double pru;          /* rpumer sily genu */
  int reseni;          /* typ reseni */
  TSVect *pr;          /* ukazatel na vl. prom. */
  TSGen *pg;           /* ukazatel na chrom. */
  TSGen *a;            /* ukazatel na aktual. chrom. */   
  TSVect *f1;          /* optimalizovana prom. */
  TSVect *f2;          /* optimalizovana prom. */
};
typedef struct TSPop TSPop;

/********************************************************/
/* typ pro vyztuz */
struct TSArmor {
  double a;            /* plocha vyztuze */
  int n;               /* pocet prutu */
};
typedef struct TSArmor TSArmor;

/********************************************************/
/* typ pro ocel */
struct TSSteel {
  int id;              /* poradove cislo */
  double fyk;          /* charakt. mez kluzu */
  double fyd;          /* navrhova mez kluzu */ 
  double es;           /* hodnota modulu pruznosti */
  double ga;           /* soucinitel spolehlivosti oceli */
  /* pro optimalizaci */
  double c1;           /* ! cena oceli za 1 kg ! */
};
typedef struct TSSteel TSSteel;

/********************************************************/
/* typ pro beton */
struct TSConcr {
  int id;              /* poradove cislo */
  double fck;          /* charakt. pevnost v tlaku */
  double fcd;          /* navrhova pevnost v tlaku */ 
  double fctm;         /* stredni pevnost v tahu */
  double fctk05;       /* 5kvantilni pevnost v tahu */
  double ecm;          /* stredni hodnota modulu pruznosti */
  double gc;           /* soucinitel spolehlivosti betonu v tlaku */
  double gcw;          /* soucinitel spolehlivosti betonu ve smyku */
  /* pro optimalizaci */
  double c1;           /* ! cena betonu za 1 m3 ! */
};
typedef struct TSConcr TSConcr;

/********************************************************/
/* typ pro bod */
struct TSNode {
  int id;              /* poradove cislo */
  double x,y;          /* souradnice uzlu */
  double f[3];         /* stycnikove zatizeni */
  int p[3];            /* uvolneni vazeb */
  int kc[3];           /* kodova cisla uzlu */
};
typedef struct TSNode TSNode;

/********************************************************/
/* typ pro FEM element */
struct TSElem {
  int id;             /* poradove cislo */
  TSNode *na,*nb;     /* ukazatele na pocatecni a koncove uzly */
  int kc[6];          /* vektor kodovych cisel */
  double f[6];        /* lokalni vektor zatizeni */
  double s[6];        /* vnitrni sily */
  double ei,ea,l;     /* prurezove charakteristiky */
  int ch;             /* kontrola MSP */
  void *pb;           /* ukazatel na nadrazeny nosnik, */
	/* nefunguje poiter. aritmetika ! */
};
typedef struct TSElem TSElem;

/********************************************************/
/* typ pro dimenzacni momentovy element */
struct TSMdim {
	/*  int bm;              znamenko momentu (0+,1-) i tazene casti */
  double m[2];           /* dimenzacni moment */
  double mu[2];          /* moment unosnosti */
  TSArmor as[2];         /* plocha vyztuze (0-dolni vlakna)*/
  double d[2];           /* rameno vnitrnich sil */
  int r[2];              /* pocet rad prutu */
  int s;                 /* pocet prislusnych FE elementu */
  double hodn[2][PROM];   /* sledovane prommene */
  int pch[2][6];         /* vysledky testu */
};
typedef struct TSMdim TSMdim;

/********************************************************/
/* typ pro dimenzacni smykovy element */
struct TSWdim {
  double q;           /* dimenzacni posouvajici sila */
  double qu;          /* smykova unosnost */
  double sw;          /* vzdalenost trminku */
  int s;              /* pocet prislusnych FE elementu */
  int pw;             /* pocet strihu */
  int pch[1];         /* vysledky testu */
};
typedef struct TSWdim TSWdim;

/********************************************************/
/* typ pro FEM nosnik */
struct TSBeam {
  int id;             /* poradove cislo */
  int ne;             /* pocet FE elementu v nosniku */
  TSNode *na,*nb;     /* pocatecni a koncovy uzel */
  TSElem *pe;         /* ukazatel na prvni pridruzeny element */
  double l;           /* delka nosniku */
  double b,h;         /* rozmery nosniku */
};
typedef struct TSBeam TSBeam;

/********************************************************/
/* typ pro dimenzacni nosnik */
struct TDimBeam {
  int id;             /* poradove cislo */
  int ne;             /* pocet FE elementu v nosniku */
  double l;           /* delka nosniku */
  double b,h;         /* rozmery nosniku */
  TSSteel *ps;        /* charakteristiky momentove oceli */
  TSSteel *psw;       /* charakteristiky smykove oceli */
  TSConcr *pc;        /* charakteristiky betonu */
  double alfa;        /* charakteristiky pro MSP */
  int pr[2];          /* prurez vyztuze z G_porfil */
  double c[2];        /* kryti vyztuze */
  int ch[2];          /* vysledky testu */
  TSMdim m[3];        /* 3 momentove elementy */
  TSWdim w[3];        /* 3 smykove elementy */
  TSBeam *pb;         /* FEM nosnik */
  TSGen *gen;         /* optim. vektor */
  /* smyk */
  int prw;            /* prurez smykove vyztuze z W_porfil */
  /* pro optimalizaci */
  TSPrice tp;         /* optimalizacni promenne */
};
typedef struct TDimBeam TDimBeam;

/********************************************************/
/* typ pro konstrukci */
struct TSKce {
  int id;             /* poradove cislo */
  int nn;             /* pocet uzlu */
  int ne;             /* pocet elementu */
  int nb;             /* pocet nosniku */
  int nc;             /* pocet neznamych */
  int nconcr;         /* pocet typu betonu */
  int nsteel;         /* pocet typu oceli */
  double *r;          /* pole pretvoreni */
  TSSteel *ps;        /* druhy oceli */
  TSConcr *pc;        /* druhy betonu */
  TSNode *pn;         /* ukazatel na uzle */
  TSElem *pe;         /* ukazatel na elementy */
  TSBeam *pb;         /* ukazatel na FE nosniky */
  TDimBeam *pdb;      /* ukazatel na dimenzacni nosniky */
};
typedef struct TSKce TSKce;

/********************************************************/
/* Globalni prommenne */
/********************************************************/
/* z rc_fce.c */
extern double G_profil[16];
extern double W_profil[4];

/********************************************************/
/* Funkce z rc_fce.c */
/********************************************************/
extern double vypocc(int prost,int pr,int prw);
extern double vypocd(int *r,double b,double h,int n,double pr,double as,double c);
extern int navrhv(double *as,double m,double b,double d,double fyk,double fyd,double fcd);
extern int pocetv(double *as,double d);
extern int checkv(double as,double b,double h,double d,double fyk);
extern int checkmsu(double m,double *mu,double as,double b,double d,double fyd,double fcd);
extern int checkxlim(double *x,double as,double b,double d,double fyd,double fck,double gc);
extern int checkpr(double b,int n,double pr,double c,int r);
extern void vypocei(double *ei,double ms,double as,double b,double h,double d,\
										double fctm,double b1,double b2,double es,double ecm,double alfa);
extern int vypocasw(double *qu,int *pn,double *s,double pr,double as,double vsd,double b,double d,\
										double fck,double fcd,double fctk05,double gc,double fywd,double romin,double smin);
extern int checkAsw(double *qu,int pn,double s,double pr,double as,double vsd,double b,double d,\
										double fck,double fcd,double fctk05,double gc,double fywd,double romin,double smin);

/********************************************************/
/* Funkce z rc_prog.c */
/********************************************************/
extern void FEtoDim(TDimBeam *pdb,TSBeam *pb);
extern void DimCheck(TDimBeam *pdb);
extern void checkmsp(TSKce *pk);
extern void kceDim(TSKce *pk);
extern void initnav(TDimBeam *pdb);
extern void initas(TDimBeam *pdb);
extern int initasw(TDimBeam *pdb);
extern void checknav(double *hodn,int *p_b,double *d,double m,double *mu,double as,double b,double h,double c,\
					  int n,double pr,double fyk,double fyd,double fck,double fcd);

/********************************************************/
/* Funkce z rc_dm.c */
/********************************************************/
extern void mtlocal(double *a,double ei,double ea,double l);
extern void addmt(double *k,double *p,int *kc,int n,int m);
extern void addvf(double *k,double *p,int *kc,int n,int m);
extern void vypocsil(TSElem *e,double *r);
extern void defmetod(TSKce *kce);

/********************************************************/
/* Funkce z rc_mat.c */
/********************************************************/
extern void reseni_rovnic(double *a,double *x,double *y,long n,long m,long as);
extern void nasmat(double *a,double *b,double *c,int m,int n,int p);

/********************************************************/
/* Funkce z rc_load.c */
/********************************************************/
extern TSSteel* TSSload(int *ns);
extern TSConcr* TSCload(int *nc);
extern TSKce* TSKload(char *jmeno);
extern void nullTDB(TDimBeam *b);

/********************************************************/
/* Funkce z rc_io.c */
/********************************************************/
extern void TSNprn(TSNode *a);
extern void TSEprn(TSElem *a);
extern void TSBprn(TSBeam *a);
extern void TDBprn(TDimBeam *a);
extern void TDBdimprn(TDimBeam *a);
extern void TSKprn(TSKce *a);
extern void TSSprn(TSSteel *a);
extern void TSCprn(TSConcr *a);
extern void TSPprn(TSPop *a);
extern void TSVprn(TSVect *a,int pocet);
extern void TSGprn(TSPop *a);
extern void TSBallprn(TDimBeam *a);
extern void TSKallprn(TSKce *a);
extern void TSPallprn(TSPop *a);
extern void TDB_OPprn(TDimBeam *a);
extern void matprn(double* a,int n,int m);
extern void B_prn(bit co);

/********************************************************/
/* Funkce z rc_alkts.c */
/********************************************************/
extern TSNode* alokaceTSN(long int a);
extern TSElem* alokaceTSE(long int a);
extern TSBeam* alokaceTSB(long int a);
extern TDimBeam* alokaceTDB(long int a);
extern TSKce* alokaceTSK(long int a);
extern TSSteel* alokaceTSS(long int a);
extern TSConcr* alokaceTSC(long int a);
extern TSPop* alokaceTSP(long int a);
extern TSVect* alokaceTSV(long int a);
extern TSGen* alokaceTSG(long int a,long int prom);
extern double* alokacer(long int a);
extern bit* alokaceb(long int a);
extern void uvolni(void *p);
extern void uvolniTSK(TSKce *a);
extern void uvolniTSP(TSPop *p);
extern void uvolniTSG(TSGen *p);

/********************************************************/
/* Funkce z rc_optim.c */
/********************************************************/
extern void OP_DBcena(TDimBeam *pdb);
extern void OP_bin_to_D(TDimBeam *pb, PopVar &pr);
extern void OP_D_to_bin(TDimBeam *pb, PopVar &pr);
extern void OP_oldtonew(TDimBeam *yng,TDimBeam *old, PopVar &p );
extern void OP_update(TSPop *p,TSGen *gkce);
extern void OP_silaPop(TSPop *p,TDimBeam *pdb,int akce);
extern void OP_silaGen(  PopVar &p, TDimBeam *pdb) ;
/*extern void OP_silaGen(TSPop *p,TDimBeam *pdb);*/
extern TSPop* OP_initVect(int pocgen,char *jm,int reseni);

/********************************************************/
/* Funkce z rc_ga.c */
/********************************************************/
extern TSPop* NewPop(int pocprom,int pocgen);
extern void InitPop(TSPop *p,int zadano);
extern void SGAIn(TSPop *p);
extern void SGAInNew(TSPop *p);
extern void SGAInNew2(TSPop *p);
extern void SGAOut(TSPop *p);
extern void SGAOut2(TSPop *p);
extern void SGAOut3(TSPop *p);
extern void G_copy(TSGen *a,TSGen *b,int pocet);
extern TSGen* G_roullette(TSGen *p_gen,int kolik,int akce);
extern void B_cross(bit *a,bit *b,int kolik);
extern void B_mut(bit *a,int kolik);
extern void InitRandom();
extern void SortPop(TSPop *p);
extern void SortPopNew(TSPop *p);
extern void RndGen(TSPop *p);

/********************************************************/
/* Funkce z rc_infi.c */
/********************************************************/
extern void InitWritePop(TSPop *p,char *jm);
extern void WritePop(TSPop *p,char *jm);
extern void EndWritePop(TDimBeam *b,char *jm);
extern void WriteLinePop(TSPop *p,FILE *f,int kg);
extern void WriteTabPop(TSPop *p,FILE *f);
extern void WriteCena(TDimBeam *a,FILE *f);
extern void WriteVyztuz(TDimBeam *a,FILE *f);

/********************************************************/
/* Funkce z rc_jm.c */
/********************************************************/
extern void LoadProstr(int argc, char *argv[ ],TProstr *p);
extern void Updatevy(TProstr *p,int cislo);

/********************************************************/
/* Funkce z rc_run.c */
/********************************************************/
extern void dimenzace(TSKce *kce);
extern void InitOpt(TSPop *pop,TSKce *kce,TSPop *gb,TProstr *o);
extern void optimalizace(TSPop *pop,TSKce *kce,TSPop *gb,TProstr *o);
extern void Optimprn(TDimBeam *beam,PopVar &p /*TSPop *pop*/,TSKce *kce);


#endif
