/*
 * rc_dm.c		ver. 1.0
 *
 * Soubor na vypocet deformacni metody
 *
 * M. Leps, 9.11.1998
 *
 */

/*#define DMDEBUG*/
/* Vyse uvedeni v makefile */

#include "rc.h"

/*
 * V pripade, ze budou fce pracovat jen uvnitr modulu
 * rc_fce.c zmenit GLOBAL na LOCAL a uvest zde prototypy
 */

/* Deklarace LOCAL funkci */

/*******************/
/* Definice funkci */
/*******************/

/***********************************************************************************************/
/* Fce na lokalni matici tuhosti */
GLOBAL void mtlocal(double *a,double ei,double ea,double l)
{
	int i;
	nulmat(i,36,a);

	/* horni trojuhelnik a diagonala */
	mtt(a,0,0) = mtt(a,3,3) = ea/l;
	mtt(a,0,3) = - mtt(a,0,0);
	mtt(a,1,1) = mtt(a,4,4) = 12*ei/(l*l*l);
	mtt(a,1,4) = - mtt(a,1,1);
	mtt(a,1,2) = mtt(a,1,5) = 6*ei/(l*l);
	mtt(a,2,4) = mtt(a,4,5) = - mtt(a,1,2);
	mtt(a,2,2) = mtt(a,5,5) = 4*ei/l;
	mtt(a,2,5) = 2*ei/l;
	/*	dolni trojuhelnik */
	mtt(a,3,0) = mtt(a,0,3);
	mtt(a,2,1) = mtt(a,5,1) = mtt(a,1,2);
	mtt(a,4,2) = mtt(a,5,4) = mtt(a,2,4);
	mtt(a,4,1) = mtt(a,1,4);
	mtt(a,5,2) = mtt(a,2,5);
	return;
}

/***********************************************************************************************/
/* Fce prida mt prvku(m*m) do mt kce(n*n) */
GLOBAL void addmt(double *k,double *p,int *kc,int n,int m)
{
  int i,j;

  if(m > n)
	 abort();

  for(i=0;i<m;i++) {
	 for(j=0;j<m;j++) {
		if ((kc[i]*kc[j]) == 0)
		  continue;
		else
		  mat(k,(kc[i]-1),(kc[j]-1),n) += mat(p,i,j,m);
	 }
  }
  
  return;
}

/***********************************************************************************************/
/* Fce prida vektor(m) do vektoru(n) */
GLOBAL void addvf(double *k,double *p,int *kc,int n,int m)
{
  int i;

  if(m > n)
	 abort();

  for(i=0;i<m;i++) {
	 if ((kc[i]) == 0)
		continue;
	 else {
		k[kc[i]-1] += p[i];
	 }
  }
  
  return;
}

/***********************************************************************************************/
/* Fce z globalnich pretvorenich(r) spocte vnitrni sily na prvku */
GLOBAL void vypocsil(TSElem *e,double *r)
{
  int i;
  double rl[6];
  double m[36];

  nulmat(i,6,rl);

  for(i=0;i<6;i++) {
	 if((e->kc[i]) == 0)
		continue;
	 else
		rl[i] = r[(e->kc[i])-1];
  }

  mtlocal(m,e->ei,e->ea,e->l);

  nasmat(m,rl,e->s,6,6,1);
  plusmat(e->s,e->f,i,6);

#ifdef DMDEBUG
  TSEprn(e);
#endif

  return;
}

/***********************************************************************************************/
/* Fce na vypocet kce deformacni metodou */

GLOBAL void defmetod(TSKce *kce)
{
   int i,j;
   int ne, nn, nc;
	double m[36];       /* matice tuhosti elementu */
	double *mt;         /* matice tuhosti cele kce */
	double *f;          /* matice prave strany */
	double *ft;         /* pomocna matice zatizeni elementu */
	TSElem *elem,*pe;   /* ukazatele na element */
	TSNode *node,*pn;   /* ukazatele na uzel */

   elem = kce->pe;
   node = kce->pn;
   ne = kce->ne;
   nn = kce->nn;
   nc = kce->nc;

	mt = alokacer(nc*nc);
	f = alokacer(nc);
	ft = alokacer(6);

	nulmat(i,nc*nc,mt);
	nulmat(i,nc,f);
	nulmat(i,nc,kce->r);

	/* vytvoreni lokalni matice tuhosti  a lokalizace */
	for(i=0;i<ne;i++) {
	  pe = elem+i;
	  mtlocal(m,pe->ei,pe->ea,pe->l);
	  addmt(mt,m,pe->kc,nc,6);
	}

#ifdef DMDEBUG
	matprn(mt,nc,nc);
#endif

	/* vytvoreni globalniho vektoru zatizeni */
	for(i=0;i<ne;i++) {
	  pe = elem+i;
	  for(j=0;j<6;j++) {
		 ft[j] = -(pe->f[j]);    /* zatizeni prvku je s - */
	  }
	  addvf(f,ft,pe->kc,nc,6);
	}

	for(i=0;i<nn;i++) {
	  pn = node+i;
	  addvf(f,pn->f,pn->kc,nc,3);
	}

#ifdef DMDEBUG
	matprn(f,nc,1);
#endif

	/* vyreseni rovnic */
	reseni_rovnic(mt,kce->r,f,nc,1,1);

#ifdef DMDEBUG
	matprn(kce->r,nc,1);
#endif

	/* vypocet vnitrnich sil */
	for(i=0;i<ne;i++) {
	  pe = elem+i;
	  vypocsil(pe,kce->r);
	}

	/* uvolnovani pameti */
	uvolni((void *) mt);
	uvolni((void *) f);
	uvolni((void *) ft);
	pe = NULL;
	pn = NULL;

	return;
}
