/*
 * rc_fce.c		ver. 1.02
 *
 * Soubor obsahujici fce pro vypocet
 * a posouzeni zelezobetonoveho nosniku
 *
 * M. Leps, 15.11.1998
 *
 */

/*#define RCDEBUG*/
/* Vyse uvedeni v makefile */

#include "rc.h"

/********************************************************/
/* definice maker */
/********************************************************/

/********************************************************/
/* vypocet vzdalenosti mezi pruty */
#define vypoca(pr) ((pr) < 0.02) ? (0.02) : (pr)
/* oriznuti dolu na 0.025 */
#define zkr025(a) ((floor((a)*40.0))/40.0)

/********************************************************/
/* definice promnennych */
/********************************************************/
GLOBAL  double G_profil[16] = {0.0055,0.006,0.0065,0.007,\
			       0.008,0.010,0.012,0.014,\
			       0.016,0.018,0.020,0.022,\
			       0.025,0.028,0.032,0.036};

GLOBAL  double G_plocha[16] = { 0.000024,\
										  0.000028,\
										  0.000033,\
										  0.000038,\
										  0.000050,\
										  0.000079,\
										  0.000113,\
										  0.000154,\
										  0.000201,\
										  0.000254,\
										  0.000314,\
										  0.000380,\
										  0.000491,\
										  0.000616,\
										  0.000804,\
										  0.001018};

GLOBAL  double W_profil[4] = {0.004,0.006,0.008,0.010};

GLOBAL  double W_plocha[16] = { 0.000013,\
										  0.000028,\
										  0.000050,\
										  0.000079};
/*
 * V pripade, ze budou fce pracovat jen uvnitr modulu
 * rc_fce.c zmenit GLOBAL na LOCAL a uvest zde prototypy
 */

LOCAL double vypocVrd2(double *ny,double b,double d,double fck,double fcd);
LOCAL double vypocVrd1(double as,double b,double d,double fctk05,double gc);
LOCAL double vypocVswd(int pw,double pr,double s,double fywd,double d);
LOCAL void vypocSmax(double *smax,double *pd,double d,double vsd,double vrd2s);
LOCAL double compx(double as,double b,double fyd,double fcd);
LOCAL void vypocpre(double *ec,double *es1,double b,double d,double as,\
							 double es,double ecm,double m);

/********************/
/* Navrh vyztuze    */
/********************/

/***************************************************************************************/
/* Vypocte kryti vyztuze */
GLOBAL double vypocc(int prost,int pr,int prw)
{
  double c;
  double cmin[2] = {0.02,0.025};

  /* c minimalni */
  if(i_pr(pr) < 0.025)
	 c = cmin[prost];
  else
	 c = i_pr(pr);

  /* trminek */
  c += w_pr(prw) + 0.005; /* + delta h */
#ifdef RCDEBUG
  printf("Kryti je %lf .\n",c);
#endif
  return(c);
}

/***************************************************************************************/
/* Vypocte rameno vnitrnich sil */
GLOBAL double vypocd(int *r,double b,double h,int n,double pr,double as,double c)
{
   double a,d,adi,sd;
	int nr,p;

	a = vypoca(pr);

	nr = (int)floor((b+a-2.*c)/(pr+a));                  /* pocet prutu v kompletni rade */
	*r = (int)ceil(((double) n)/((double) nr));         /* pocet rad */
	if(*r == 0) {
	  return(0);
	}
	else {
	  p = n-((*r)-1)*nr;                             /* pocet prutu v posledni rade */
	  sd = ((*r)-1)*((h-c+a+pr/2)-(a+pr)*(*r)/2);    /* soucet ramen n-1 rad */
	  d = h-c-pr/2+(1-(*r))*(a+pr);                  /* rameno posledni rady */
	  adi = (npr_as(nr,pr))*sd+(npr_as(p,pr))*d;     /* suma Aidi */
	  d = adi/as;                                    /* teziste vyztuze */

#ifdef RCDEBUG
	  printf("%lf %d %d %d %d %lf %lf %lf\n",a,n,nr,*r,p,sd,adi,as);
	  printf("Rameno sily ve vyztuzi je %lf :\n\n",d);
#endif
	}
	return(d);
}

/***************************************************************************************/
/* Fce na vypocet potrebne plochy vyztuze */
GLOBAL int navrhv(double *as,double m,double b,double d,double fyk,double fyd,double fcd)
{
	double det;
	int b_tmp;

	det = 4*d*d-8*m/(fcd*b);
#ifdef RCDEBUG
	printf("Determinant je %lf .\n",det);
#endif

	if (det < 0) {
#ifdef RCDEBUG
	printf("Prilis veliky moment, nutno zvetsit prurez. \n");
#endif
		return(ERRLOWC);
	}

#ifdef RCDEBUG
	*as = (2*d+sqrt(det))/((2*fyd)/(fcd*b));
	printf("Prvni hodnota je %lf .\n",as);
#endif

	*as = (2*d-sqrt(det))/((2*fyd)/(fcd*b))*1.005;
 /* navyseni o 0.5%, nebot v nekterych pripadech nevysl MSU */

	b_tmp = checkv(*as,b,(d+0.025),d,fyk);

	if (b_tmp == ERRLOWS) {
	   *as = (600*(b*d)/fyk);
	}
	
#ifdef RCDEBUG
	printf("Predbezna hodnota vyztuze je %lf .\n\n",*as);
#endif
	return(GOOD);
}

/***************************************************************************************/
/* Fce na urceni poctu prutu */
GLOBAL int pocetv(double *as,double d)
{
  int n;
  double np;

  np = 4*(*as)/(MPI*d*d);
  n = (int)ceil(np);
  if(n == 1)
	 n = 2;
  *as = npr_as(n,d);

#ifdef RCDEBUG
  printf("Vychazi pocet prutu: %lf .\n",np);
  printf("Navrh poctu prutu: %d .\n",n);
  printf("Konecna hodnota plochy vyztuze je %lf .\n\n",*as);
#endif
  return n;
}

/********************/
/* Kontrola dle MSU */
/********************/

/***************************************************************************************/
/* Fce na kontrolu stupne vyztuzeni */
GLOBAL int checkv(double as,double b,double h,double d,double fyk)
{
	/* pozor .. fyk ve vzorci v [MPA], zadani v [kPa] */

	double amin,amax;

	amin = (600/fyk)*b*d;
	amax = 0.04*b*h;
#ifdef RCDEBUG
	printf("MAX=%lf ?> as=%lf ?> MIN=%lf .\n",amax,as,amin);
#endif

	if (as > amax)	{				/* moc vyztuze, malo betonu */
#ifdef RCDEBUG
	printf("Prilis mnoho vyztuze, nutno zvetsit prurez .\n\n");
#endif
		return(ERRLOWC);
	}
	else if (as < amin) {		/* moc betonu, malo vyztuze */
#ifdef RCDEBUG
	printf("Prilis mnoho betonu, nutno zvetsit plochu vyztuze .\n\n");
#endif
		return(ERRLOWS);
	}
	else {
#ifdef RCDEBUG
	printf("Vse v poradku .\n\n");
#endif
		return(GOOD);
	}
}

/***************************************************************************************/
/* Fce na posudek momentu v MSU */
GLOBAL int checkmsu(double m,double *mu,double as,double b,double d,double fyd,double fcd)
{
   double x;

	x = compx(as,b,fyd,fcd);

	*mu = as*fyd*(d-0.416*x);

#ifdef RCDEBUG
	printf("mu=%lf ?> ms=%lf .\n",*mu,m);
#endif

	if (m > (*mu)) {
#ifdef RCDEBUG
	printf("Prurez nevyhovel .\n\n");
#endif
	   return(BAD);
	}	   

#ifdef RCDEBUG
	printf("Vse v poradku .\n\n");
#endif
	return(GOOD);
}

/***************************************************************************************/
/* Fce na posudek prevyztuzeni dle xlim */
GLOBAL int checkxlim(double *x,double as,double b,double d,double fyd,double fck,double fcd)
{
   /* fyd [kPa], modul pruznosti oceli Es se uvazuje 200GPa */

   double xlim;

	*x = compx(as,b,fyd,(fcd));
	xlim = d*0.0035/(0.0035+(fyd/200000000.));

#ifdef RCDEBUG
	printf("x=%lf <? xlim=%lf ",*x,xlim);
#endif

	/* vypusteni zakazanych hodnot */
	if (*x > xlim) {
#ifdef RCDEBUG
	printf("\nPrurez prevyztuzen .\n\n");
#endif
	   return(ERRLOWC);
	}

#ifdef RCDEBUG
	printf("... vyhovelo .\n");
#endif

	/* EC2 vyzadovane hodnoty */
	/* Lze vypustit s varovanim ? */
	if (fck > 35) {
	   if ((*x/d) > 0.35) {
#ifdef RCDEBUG
	printf("ale podle EC2 prurez prevyztuzen .\n\n");
#endif
		   return(ERRLOWCEC);
		}
	}
	else {
	  if ((*x/d) > 0.45) {
#ifdef RCDEBUG
	printf("ale podle EC2 prurez prevyztuzen .\n\n");
#endif
		   return(ERRLOWCEC);
	  }
	}

#ifdef RCDEBUG
	printf("Vse v poradku .\n\n");
#endif
	return(GOOD);
}

/***************************************************************************************/
/* Zda se vejdou pruty do prurezu v jedne rade */
GLOBAL int checkpr(double b,int n,double pr,double c,int r)
{
   double br,a;
	int nr;

	a = vypoca(pr);                                /* svetla vzdalenost mezi pruty */
	nr = (int)floor((b+a-2*c)/(pr+a));                  /* pocet prutu v kompletni rade */
	n -= ((r-1)*nr);                               /* pocet prutu v posledni rade ... */
	br = 2*c+n*pr+(n-1)*a;                         /* a misto ktere zaberou */

#ifdef RCDEBUG
	printf("br=%lf <? b=%lf ",br,b);
#endif

	if (br > b) {
#ifdef RCDEBUG
	printf("Nevyhovelo .\n\n");
#endif
	   return(BAD);
	}
#ifdef RCDEBUG
	printf("Vse v poradku .\n\n");
#endif
	return(GOOD);
}

/***************************************************************************************/
/* Fce na vypocet x */
LOCAL double compx(double as,double b,double fyd,double fcd)
{
   double x;
	x = (as*fyd)/(0.81*b*(fcd));

#ifdef RCDEBUG
	printf("Hodnota x je %lf .\n\n",x);
#endif
	return(x);
}

/********************/
/* Kontrola dle MSP */
/********************/

/***************************************************************************************/
/* Fce na vypocet pretvoreni dle MSP */
LOCAL void vypocpre(double *ec,double *es1,double b,double d,double as,\
							 double es,double ecm,double m)
{
  double x,ae,be,dx;

	ae = as*es;
	be = b*ecm;
	x = (sqrt(ae*ae+2*be*d*ae)-ae)/be;
	dx = d-x;
	*es1 = -m*dx/(be*x*x*x/6-ae*d*dx);
	*ec = (*es1)*x/dx;

#ifdef RCDEBUG
	printf("Hodnoty x,es,ec jsou %lf %lf %lf .\n\n",x,*es1,*ec);
#endif
	return;
}

/***************************************************************************************/
/* Fce na vypocet momentu pri vzniku trhlin */
GLOBAL void vypocei(double *ei,double ms,double as,double b,double h,double d,\
						 double fctm,double b1,double b2,double es,double ecm,double alfa)
{
   double ac,agi,ii,sum;
   double ksi,mr,k1,k2,km,ec,es1;

/* vypocet idealnich velicin */
	ac = b*h;
	sum = ac+alfa*as;

	if (sum <= 0) {
#ifdef RCDEBUG
	printf("Nulova nebo zaporna hodnota v rozmerech nebo vyztuzi !\n\n");
#endif
	   abort();
	}

	agi = (0.5*ac*h+as*alfa*d)/sum;
	ii = (ac*h*h/12)+(ac*(agi-h*0.5)*(agi-h*0.5))+(alfa*as*(d-agi)*(d-agi));
	mr = (fctm*ii)/(h-agi);

#ifdef RCDEBUG
	printf("Hodnoty agi,ii,mkrit,ms jsou %lf %lf %lf %lf .\n",agi,ii,mr,ms);
#endif

	if (mr > ms) {
#ifdef RCDEBUG
	printf("Nedojde ke vzniku trhlin. \n");
#endif
	   *ei = ecm*ii;
	   return;
	}

	ksi = 1-b1*b2*(mr/ms)*(mr/ms);
	vypocpre(&ec,&es1,b,d,as,es,ecm,ms); /* vypocet pretvoreni pri ms */
	k1 = mr/(ecm*ii);                    /* krivost ve stavu 1 */
	k2 = (ec+es1)/d;                     /* krivost ve stavu 2 */
	km = (1-ksi)*k1+ksi*k2;              /* prumerna krivost */
	*ei = ms/km;                         /* vysledna tuhost */

#ifdef RCDEBUG
	printf("Hodnoty k1,k2,ec,es1 jsou %lf %lf %lf %lf .\n",k1,k2,ec,es1);
	printf("Hodnoty ksi,km,ei jsou %lf %lf %lf .\n\n",ksi,km,*ei);
#endif
	return;
}

/*************************/
/* Navrh smykove vyztuze */
/*************************/

/***************************************************************************************/
/* Fce na vypocet smykove unosnosti */
LOCAL double vypocVrd2(double *ny,double b,double d,double fck,double fcd)
{
  *ny = 0.7-fck/200000.0;
  *ny = (*ny < 0.5) ? (0.5) : (*ny);
  return(0.5*(*ny)*fcd*b*0.9*d);
}

LOCAL double vypocVrd1(double as,double b,double d,double fctk05,double gc)
{
  double trd,k,ro;
  trd = 0.25*fctk05/gc;
  k = 1.6-d;
  k = (k < 1.0) ? (1.0) : (k);
  ro = as/(b*d);
  ro = (ro > 0.02) ? (0.02) : (ro);
  return(trd*k*(1.2+40*ro)*b*d);
}

LOCAL double vypocVswd(int pw,double pr,double s,double fywd,double d)
{
  return(npr_as(pw,pr)*fywd*0.9*d/s);
}

LOCAL void vypocSmax(double *smax,double *pd,double d,double vsd,double vrd2s)
{
  /* smax,smaxt(pd) dle NAD */
  if(vsd <= 0.2*vrd2s) {
	 *smax = zkr025(0.8*d);
	 *smax = (*smax > 0.4) ? (0.4) : (*smax);
	 *pd = d;
	 *pd = (*pd > 0.8) ? (0.8) : (*pd);
  }
  else if(vsd < 0.66667*vrd2s) {
	 *smax = zkr025(0.6*d);
	 *smax = (*smax > 0.3) ? (0.3) : (*smax);
	 *pd = 0.8*d;
	 *pd = (*pd > 0.4) ? (0.4) : (*pd);
  }
  else {
	 *smax = zkr025(0.3*d);
	 *smax = (*smax > 0.2) ? (0.2) : (*smax);
	 *pd = 0.6*d;
	 *pd = (*pd > 0.3) ? (0.3) : (*pd);
  }
  return;
}

/* Fce na vypocet smykove vyztuze */
GLOBAL int vypocasw(double *qu,int *pn,double *s,double pr,double as,double vsd,double b,double d,\
						 double fck,double fcd,double fctk05,double gc,double fywd,double romin,double smin)
{
  double pd,ro,vswd,ny;
  double asw,romax,smax,vrd1,vrd2s,dvsd;
  int btmp;

  vrd2s = vypocVrd2(&ny,b,d,fck,fcd);
#ifdef RCDEBUG
	 printf("Vsd=%lf < Vrd2s=%lf ? ",vsd,vrd2s);
#endif

  if(vsd > vrd2s) {
#ifdef RCDEBUG
	 printf("Nevyhovelo, drti se beton. \n");
#endif
	 return(BAD);
  }
  else {
#ifdef RCDEBUG
	 printf("OK \n");
#endif
  }

  /* smax,smaxt(pd) dle NAD */
  vypocSmax(&smax,&pd,d,vsd,vrd2s);

  romax = 0.5*ny*fcd/fywd;             /* maximalni stupen vyztuzeni */
  *pn = 2*(int)ceil((b/pd+1.0)/2);     /* pocet strihu */
  asw = npr_as(*pn,pr);                /* plocha smykove vyztuze */
  vrd1 = vypocVrd1(as,b,d,fctk05,gc);
  dvsd = vsd-vrd1;                     /* posouvajici sila prenesena trminky */

#ifdef RCDEBUG
  printf("Vsd=%lf < Vrd1=%lf \n",vsd,vrd1);
#endif

  if(dvsd <= 0.0) {
	 *s = zkr025(asw/(romin*b));
#ifdef RCDEBUG
	 printf("Neni nutna smykova vyztuz, pouze konstr. v. :\n");
	 printf("pr %3lf a %4lf - %1d strihy .\n",(pr)*1000,(*s)*1000,*pn);
#endif
  }
  else {
	 *s = zkr025(asw*fywd*0.9*d/dvsd);
#ifdef RCDEBUG
	 printf("Smykova vyztuz :\n");
	 printf("pr %3lf a %4lf - %1d strihy .\n",(pr)*1000,(*s)*1000,*pn);
#endif
  }

  do {
	 ro = asw/((*s)*b);
	 btmp = GOOD;
	 if(*s < smin) {
#ifdef RCDEBUG
		printf("Male s .. zvetsit profil.\n");
#endif
		return(ZMENA);
	 }
	 else if(*s > smax) {
		*s = smax;
		btmp = BAD;
#ifdef RCDEBUG
		printf("Velke s.\n");
#endif
	 }
	 else if(ro < romin) {
		*s = zkr025(asw/(romin*b));
		btmp = BAD;
#ifdef RCDEBUG
		printf("Male ro.\n");
#endif
	 }
	 else if(ro > romax) {
		*s = zkr025(asw/(romax*b));
		btmp = BAD;
#ifdef RCDEBUG
		printf("Velke ro.");
#endif
	 }
  } while(!btmp);

#ifdef RCDEBUG
  printf("pr %3lf a %4lf - %1d strihy .\n",(pr)*1000,(*s)*1000,*pn);
#endif
  vswd = vypocVswd(*pn,pr,*s,fywd,d);
  *qu  = vrd1 + vswd;
  return(GOOD);
}

/* Fce na kontrolu smykove vyztuze */
GLOBAL int checkAsw(double *qu,int pn,double s,double pr,double as,double vsd,double b,double d,\
						 double fck,double fcd,double fctk05,double gc,double fywd,double romin,double smin)
{
  double ny,pd,vswd,ro;
  double romax,smax,vrd1,vrd2s;

  vrd2s = vypocVrd2(&ny,b,d,fck,fcd);
  if(vsd > vrd2s) {
	 *qu = vrd2s;
	 return(BAD);
  }

  /* smax,smaxt(pd) dle NAD */
  vypocSmax(&smax,&pd,d,vsd,vrd2s);
  romax = 0.5*ny*fcd/fywd;                 /* maximalni stupen vyztuzeni */
  ro = npr_as(pn,pr)/(s*b);                /* smykove vyztuzeni */

  vrd1 = vypocVrd1(as,b,d,fctk05,gc);
  vswd = vypocVswd(pn,pr,s,fywd,d);
  *qu  = vrd1 + vswd;
  if(ro < romin) {
	 return(BAD);
  }
  else if(ro > romax) {
	 return(BAD);
  }
  else if(s < smin) {
	 return(BAD);
  }
  else if(s > smax) {
	 return(BAD);
  }
  /* pricna vzdalenost ramen trminku od sebe */
  else if(pd < b/(pn-1)) {
	 return(BAD);
  }
  else if(0.1 > b/(pn-1)) {
	 return(BAD);
  }
  return(GOOD);
}
