/*
 * rc_ga.c		ver. 1.01
 *
 * soubor s funkcemi pro geneticky algoritmus
 *
 * M. Leps, 3.12.1998
 *
 */

#include "rc.h"

/* makra pro typ rulety */
#define KLADNA        0             /* normalni ruleta (vybere nejsilnejsi gen) */
#define INVERZNI      1             /* inverzni ruleta (vybere nejslabsi gen) */

#define PLUS          0
#define MINUS         1

#define PMUTATION     0.03        /* pravdepodobnost mutace */

/*
 * V pripade, ze budou fce pracovat jen uvnitr modulu
 * zmenit GLOBAL na LOCAL a uvest zde prototypy
 */

/* Deklarace LOCAL funkci */
/*LOCAL void InitRandom();*/
LOCAL TSVect* F_roullette(TSVect *pv,int kolik);
LOCAL void F_change(TSVect *all,TSVect *pv,int kolik,int akce);
LOCAL int SortSila(const void *pa, const void *pb);

/*******************/
/* Definice funkci */
/*******************/

/***********************************************************************************************/
/* Gen. algoritmus */
/* Alokace pameti */
GLOBAL TSPop* NewPop(int pocprom,int pocgen)
{
  int i, j;
  TSPop *p_pop;

  p_pop = alokaceTSP(1);
  p_pop->npr = pocprom;
  p_pop->pr = alokaceTSV(p_pop->npr);
  p_pop->it = 0;
  p_pop->ngen = pocgen;
  /* !! alokuje o 2 geny navic, pro krizeni !! */
  p_pop->pg = alokaceTSG(p_pop->ngen+2,p_pop->npr);

  for(i=0;i<p_pop->npr;i++) {
    p_pop->pr[i].id = i+1;
    p_pop->pr[i].n = 0;
    p_pop->pr[i].f = 0.0;
    p_pop->pr[i].opt = 0;
  }

  /* !! alokuje o 2 geny navic, pro krizeni !! */
  for(i=0;i<(p_pop->ngen+2);i++) {
    p_pop->pg[i].id = i+1;
    p_pop->pg[i].f = 0.0;
    p_pop->pg[i].c = 0.0;
    for(j=0;j<p_pop->npr;j++) {
      p_pop->pg[i].bin[j] = 0;
    }
  }
  return(p_pop);
}

/* Inicializace */
GLOBAL void InitPop(TSPop *p,int zadano)
{
  int i, j, kde;

  if(zadano < 1) {
    for(i=0;i<p->ngen;i++) {
      seta(p,i);
      RndGen(p);
    }
  }
  else {
    for(i=zadano;i<p->ngen;i++) {
      if(srnd() < 0.2) { /* pravd. vyberu noveho chromozomu */
	seta(p,i);
	RndGen(p);
      }
      else {
	for(j=0;j<p->npr;j++) {
	  if((p->pr+j)->opt == 1) {
	    kde = rnd(i);
	    p->pg[i].bin[j] = p->pg[kde].bin[j];
	    kde = rnd(i);
	    /*B_cross(p->pg[i].bin+j,p->pg[kde].bin+j,p->pr[j].n);*/
	    p->pg[i].bin[j] = p->pg[kde].bin[j];
	    if(srnd() < 0.3) { /* pravd. mutace */
	      B_mut(p->pg[i].bin+j,p->pr[j].n);
	    }
	  }
	}
      }
    }
  }
  return;
}

/* Optimalizace-vytvoreni novych dvou genu */
GLOBAL void SGAIn(TSPop *p)
{
  int i;
  TSGen *p1,*p2,*n1,*n2;

  p1 = G_roullette(p->pg,p->ngen,KLADNA);
  p2 = G_roullette(p->pg,p->ngen,KLADNA);
  n1 = p->pg+(p->ngen+0);
  n2 = p->pg+(p->ngen+1);
  G_copy(n1,p1,p->npr);
  G_copy(n2,p2,p->npr);

/*  p->f1 = F_roullette(p->pr,p->npr);
  p->f2 = F_roullette(p->pr,p->npr);*/

  for(i=0;i<p->npr;i++) {
    if((p->pr+i)->opt == 1) {
		B_cross(n1->bin+i,n2->bin+i,p->pr[i].n);
      if(srnd() < PMUTATION)
		  B_mut(n1->bin+i,p->pr[i].n);
      if(srnd() < PMUTATION)
		  B_mut(n2->bin+i,p->pr[i].n);
    }
  }
  return;
}

GLOBAL void SGAInNew(TSPop *p)
{
  int i;
  double t;
  TSGen *p1,*p2,*n1,*n2;

  p1 = G_roullette(p->pg,p->ngen,KLADNA);
  p2 = G_roullette(p->pg,p->ngen,KLADNA);
  n1 = p->pg+(p->ngen+0);
  n2 = p->pg+(p->ngen+1);
  G_copy(n1,p1,p->npr);
  G_copy(n2,p2,p->npr);

/*  p->f1 = F_roullette(p->pr,p->npr);
  p->f2 = F_roullette(p->pr,p->npr);*/

  t = 0.51;
  for(i=0;i<p->npr;i++) {
    if((p->pr+i)->opt == 1) {
      if(srnd() < t) {
	B_cross(n1->bin+i,n2->bin+i,p->pr[i].n);
	if(srnd() < PMUTATION)
	  B_mut(n1->bin+i,p->pr[i].n);
	if(srnd() < PMUTATION)
	  B_mut(n2->bin+i,p->pr[i].n);
      }
    }
  }
  return;
}

GLOBAL void SGAInNew2(TSPop *p)
{
  bit tmp;
  int i,j;
  TSGen *p1,*p2,*n1,*n2;

  p1 = G_roullette(p->pg,p->ngen,KLADNA);
  p2 = G_roullette(p->pg,p->ngen,KLADNA);
  n1 = p->pg+(p->ngen+0);
  n2 = p->pg+(p->ngen+1);
  G_copy(n1,p1,p->npr);
  G_copy(n2,p2,p->npr);

  p->f1 = F_roullette(p->pr,p->npr);
  p->f2 = F_roullette(p->pr,p->npr);

  i = p->f1->id;
  B_cross(n1->bin+i,n2->bin+i,p->pr[i].n);
  for(j=i+1;j<p->npr;j++) {
	 tmp = n1->bin[j];
	 n1->bin[j] = n2->bin[j];
	 n2->bin[j] = tmp;
    	 if(srnd() < PMUTATION)
		B_mut(n1->bin+j,p->pr[j].n);
	 if(srnd() < PMUTATION)
		B_mut(n2->bin+j,p->pr[j].n);
  }
  return;
}

/* Optimalizace-nahrada dvou starych genu */
GLOBAL void SGAOut(TSPop *p)
{
  double t;
  TSGen *p1,*p2,*n1,*n2 ;

  n1 = p->pg+(p->ngen+0);
  n2 = p->pg+(p->ngen+1);
  p1 = p->pg+(n1->id-1);
  p2 = p->pg+(n2->id-1);
  if(n1->f < p1->f) {
    p1 = G_roullette(p->pg,p->ngen,INVERZNI);
    G_copy(p1,n1,p->npr);
/*    F_change(p->pr,p->f1,p->npr,PLUS);
    F_change(p->pr,p->f2,p->npr,PLUS);*/
  }
  else {
    t = (1.01-(p->it/IT));
    if(srnd() < (-(n1->f - (p1->f))/t))
      G_copy(p1,n1,p->npr);
/*    F_change(p->pr,p->f1,p->npr,MINUS);
    F_change(p->pr,p->f2,p->npr,MINUS);*/
  }
  if(n2->f < p2->f) {
    p2 = G_roullette(p->pg,p->ngen,INVERZNI);
    G_copy(p2,n2,p->npr);
/*    F_change(p->pr,p->f1,p->npr,PLUS);
    F_change(p->pr,p->f2,p->npr,PLUS);*/
  }
  else {
    t = (1.01-(p->it/IT));
    if(srnd() < (-(n1->f - p1->f)/t))
      G_copy(p1,n1,p->npr);
/*    F_change(p->pr,p->f1,p->npr,MINUS);
    F_change(p->pr,p->f2,p->npr,MINUS);*/
  }
  return;
}
/* Optimalizace-nahrada dvou starych genu */
GLOBAL void SGAOut2(TSPop *p)
{
  TSGen *n1,*n2,*o1,*o2;
  double t;

  n1 = p->pg+(p->ngen+0);
  n2 = p->pg+(p->ngen+1);
  o1 = G_roullette(p->pg,p->ngen,INVERZNI);
  if(n1->f < o1->f) {
    G_copy(o1,n1,p->npr);
/*    F_change(p->pr,p->f1,p->npr,PLUS);
    F_change(p->pr,p->f2,p->npr,PLUS);*/
  }
  else {
    t = (1.01-(p->it/IT));
    if(srnd() < (-(n1->f - o1->f)/t))
      G_copy(o1,n1,p->npr);
      
/*    F_change(p->pr,p->f1,p->npr,MINUS);
    F_change(p->pr,p->f2,p->npr,MINUS);*/
  }
  o2 = G_roullette(p->pg,p->ngen,INVERZNI);
  if(n2->f < o2->f) {
    G_copy(o2,n2,p->npr);
/*    F_change(p->pr,p->f1,p->npr,PLUS);
    F_change(p->pr,p->f2,p->npr,PLUS);*/
  }
  else {
    t = (1.01-(p->it/IT));
    if(srnd() < (-(n1->f - o1->f)/t))
      G_copy(o1,n1,p->npr);
/*    F_change(p->pr,p->f1,p->npr,MINUS);
    F_change(p->pr,p->f2,p->npr,MINUS);*/
  }
  return;
}
/* Optimalizace-nahrada prvniho genu */
GLOBAL void SGAOut3(TSPop *p)
{
  TSGen *n1,*n2,*o;

  n1 = p->pg+(p->ngen+0);
  n2 = p->pg+(p->ngen+1);
  o = p->pg+0;
  if(n1->f < o->f) {
    G_copy(o,n1,p->npr);
/*    F_change(p->pr,p->f1,p->npr,PLUS);
    F_change(p->pr,p->f2,p->npr,PLUS);*/
  }
  else {
/*    F_change(p->pr,p->f1,p->npr,MINUS);
    F_change(p->pr,p->f2,p->npr,MINUS);*/
  }
  if(n2->f < o->f) {
    G_copy(o,n2,p->npr);
/*    F_change(p->pr,p->f1,p->npr,PLUS);
    F_change(p->pr,p->f2,p->npr,PLUS);*/
  }
  else {
/*    F_change(p->pr,p->f1,p->npr,MINUS);
    F_change(p->pr,p->f2,p->npr,MINUS);*/
  }
  return;
}


/***********************************************************************************************/
/* Fce pro vyber genu ruletou */
GLOBAL TSGen* G_roullette(TSGen *pg,int kolik,int akce)
{
  int i;
  double suma = 0.0;
  double interval = 0.0;
  double kde;
  TSGen *p_ch;

  kde = srnd();
  if(akce == KLADNA) { /* vetsi pravdepodobnost pro mensi f */
    for(i=0;i<kolik;i++) {
      suma += (pg+i)->f1; /*(1.0/((pg+i)->f+0.0000001));*/
    }
    for(i=0;i<kolik;i++) {
      p_ch = pg+i;
      interval += (pg+i)->f1/suma; /*(1.0/(p_ch->f+0.0000001))/suma;*/
      if(interval > kde)
	break;
    }
  }
  else { /* vetsi pravdepodobnost pro vetsi f */
    for(i=0;i<kolik;i++) {
      suma += (pg+i)->f;
    }
    for(i=0;i<kolik;i++) {
      p_ch = pg+i;
      interval += (p_ch->f)/suma;
      if(interval > kde)
	break;
    }
  }
  return(p_ch);
}

/* Fce pro vyber funkce ruletou */
LOCAL TSVect* F_roullette(TSVect *pv,int kolik)
{
  int i;
  double suma = 0.0;
  double interval = 0.0;
  double kde;
  TSVect *pt;

  kde = srnd();
  for(i=0;i<kolik;i++) {
    if((pv+i)->opt == 1)
      suma += (pv+i)->f;
  }
  for(i=0;i<kolik;i++) {
    pt = pv+i;
    if(pt->opt == 1)
      interval += (pt->f)/suma;
    if(interval > kde)
      break;
  }
  return(pt);
}

/* uprava sily promenne */
LOCAL void F_change(TSVect *all,TSVect *pv,int kolik,int akce)
{
  int i;
  int s = 0;
  double v;
  double oco[2] = { -0.02, 0.001};

  for(i=0;i<kolik;i++) {
    if(((all+i)->opt == 1) && ((all+i)->id != pv->id)) {
      v = (all+i)->f + oco[akce];
      if(v > 0.) {
	(all+i)->f = v;
	s++;
      }
    }
  }
  v = (pv->f)-(s*oco[akce]);
  if(v > 0.)
    pv->f = v;
  return;
}

/***********************************************************************************************/
/* Kopie Genu */
GLOBAL void G_copy(TSGen *a,TSGen *b,int pocet)
{
  int i;

  a->id = b->id; 
  a->f = b->f; 
  a->c = b->c; 
  for(i=0;i<pocet;i++) {
    a->bin[i] = b->bin[i];
  }
  return;
}

/***********************************************************************************************/
/* Fce pro krizeni */
GLOBAL void B_cross(bit *a,bit *b,int kolik)
{
  int kde;
  bit o,p,m,n;

  kde = rnd(kolik+1);
  o = *a;
  p = *b;
  m = ~0x0;
  m <<= kde;
  n = ~m;
  *a = (o&m)|(p&n);
  *b = (o&n)|(p&m);
  return;
}

/* Fce pro mutaci */
GLOBAL void B_mut(bit *a,int kolik)
{
  int kde;
  bit m;

  kde = rnd(kolik);
  m = 1;
  m <<= kde;
  *a = ((~m)&(*a))|(m&(~(*a)));
  return;
}

/***********************************************************************************************/
/* Nahodna cisla */
GLOBAL void InitRandom()
{
  time_t t;

  t = time(&t);
  //srand48(t);
  srand(t);
}

/***********************************************************************************************/
/* Utrideni genu dle sily */
GLOBAL void SortPop(TSPop *p)
{
  int i,j,min;
  double sum = 0.0;
  TSGen *tmp;

  tmp =  p->pg+(p->ngen+0);

  for(i=0;i<(p->ngen-1);i++) {
    min = i;
    for(j=(i+1);j<p->ngen;j++) {
      if((p->pg+min)->f > (p->pg+j)->f) {
	min = j;
      }
    }
    if(i != min) {
      G_copy(tmp,p->pg+min,p->npr);
      G_copy(p->pg+min,p->pg+i,p->npr);
      G_copy(p->pg+i,tmp,p->npr);
    }
  }
  for( i=0;i<p->ngen;i++ ){
    sum += (p->pg+i)->f;
  }
  p->pru = sum/p->ngen;
  return;
}

GLOBAL void SortPopNew(TSPop *p)
{
  int i,kde;
  double sum = 0.0;
  TSGen *tmp;
  tmp =  p->pg+(p->ngen+0);
  kde = 0;
  for(i=1;i<p->ngen;i++) {
    if((p->pg+i)->f < (p->pg+kde)->f)
      kde = i;
  }
  G_copy(tmp,p->pg+0,p->npr);
  G_copy(p->pg+0,p->pg+kde,p->npr);
  G_copy(p->pg+kde,tmp,p->npr);

  for( i=0;i<p->ngen;i++ ){
    sum += (p->pg+i)->f;
  }
  p->pru = sum/p->ngen;
  /*  qsort((void *)p->pg,p->ngen,sizeof(TSGen),SortSila);*/
}

LOCAL int SortSila(const void *pa, const void *pb)
{
  TSGen *a = (TSGen *) pa;
  TSGen *b = (TSGen *) pb;

  if((a->f) < (b->f))
    return(-1);
  else if((a->f) == (b->f))
    return(0);
  else
    return(1);
}

/***********************************************************************************************/
/* Vytvoreni nahodneho genu */
GLOBAL void RndGen(TSPop *p)
{
  int i;

  for(i=0;i<p->npr;i++) {
    if((p->pr+i)->opt == 1)
      p->a->bin[i] = rnd(~((-1)<<(p->pr[i].n)));
  }
  return;
}
