/*
 * rc_io.c		ver. 1.01
 *
 * Soubor s I/O funkcemi pro RC
 *
 * M. Leps, 4.12.1998
 *
 */

#include "rc.h"

/* Deklarace LOCAL funkci */

/*******************/
/* Definice funkci */
/*******************/

/***********************************************************************************************/
/* Fce pro vypis promennych */
/* Vypis uzlu */
GLOBAL void TSNprn(TSNode *a)
{
  printf("Uzel c.%ld : x= %lf , y= %lf \n",a->id,a->x,a->y);
  printf("Uvolneni vazeb : %ld %ld %ld \n",a->p[0],a->p[1],a->p[2]);
  printf("Kodova cisla : %ld %ld %ld \n",a->kc[0],a->kc[1],a->kc[2]);
  printf("Stycnikove zatizeni : %lf %lf %lf \n",a->f[0],a->f[1],a->f[2]);
  return;
}

/* Vypis FEM elementu */
GLOBAL void TSEprn(TSElem *a)
{
  int i;

  printf("Element c.%ld : \n",a->id);
  printf("Kodova cisla : %ld %ld %ld %ld %ld %ld \n",\
			a->kc[0],a->kc[1],a->kc[2],a->kc[3],a->kc[4],a->kc[5]);
  printf("Lokalni vektor zatizeni: ");
  for(i=0;i<6;i++) {
	 printf("%lf ",a->f[i]);
  }
  printf("\nVnitrni sily: ");
  for(i=0;i<6;i++) {
	 printf("%lf ",a->s[i]);
  }
  printf("\n");
  return;
}

/* Vypis FE nosniku */
GLOBAL void TSBprn(TSBeam *a)
{
  printf("Nosnik c.%ld : %4.0lfx%3.0lf \n",a->id,(a->b)*1000,(a->h)*1000);
  printf("delka: %lf \n",a->l);
  printf("Pocet FEM elementu : %ld \n\n",a->ne);
  return;
}

/* Vypis nosniku */
GLOBAL void TDBprn(TDimBeam *a)
{
  printf("Nosnik c.%ld : %4.0lfx%3.0lf \n",a->id,(a->b)*1000,(a->h)*1000);
  printf("delka: %lf \n",a->l);
  printf("Typ oceli : %ld \n",a->ps->id);
  printf("Typ betonu : %ld \n",a->pc->id);
  printf("Trminky : %ld \n",a->psw->id);
  printf("Pocet FEM elementu : %ld \n\n",a->ne);
  return;
}

/* Vypis dimenzace nosniku */
GLOBAL void TDBdimprn(TDimBeam *a)
{
  WriteVyztuz(a,stdout);
  return;
}

/* Vypis konstrukce */
GLOBAL void TSKprn(TSKce *a)
{
  printf("Vypis hodnot kce : \n");
  printf("Pocet uzlu : %ld \n",a->nn);
  printf("Pocet elementu : %ld \n",a->ne);
  printf("Pocet nosniku : %ld \n",a->nb);
  printf("Pocet neznamych : %ld \n\n",a->nc);
  return;
}

/* Vypis oceli */
GLOBAL void TSSprn(TSSteel *a)
{
  printf("Ocel c.%ld: \n",a->id);
  printf("fyk,Ga,fyd : %lf %lf %lf \n",a->fyk,a->ga,a->fyd);
  printf("Es : %lf \n",a->es);
  printf("Cena : %lf Kc/kg \n\n",a->c1);
  return;
}

/* Vypis betonu */
GLOBAL void TSCprn(TSConcr *a)
{
  printf("Beton c.%ld: \n",a->id);
  printf("fck,Gc,fcd : %lf %lf %lf \n",a->fck,a->gc,a->fcd);
  printf("fctm,fctk05,gcw : %lf %lf %lf \n",a->fctm,a->fctk05,a->gcw);
  printf("Ecm : %lf \n",a->ecm);
  printf("Cena : %lf Kc/m3 \n\n",a->c1);
  return;
}

/* Vypis Populace */
GLOBAL void TSPprn(TSPop *a)
{
  printf("Populace :\n");
  printf("Pocet promennych: %ld \n",a->npr);
  printf("Pocet chromozomu: %ld \n",a->ngen);
  printf("\n");
  return;
}

/* Vypis Promennych */
GLOBAL void TSVprn(TSVect *a,int pocet)
{
  int i;
  TSVect *v;

  printf("Promenne :\n");
  printf("Id  b     sila    opt \n");
  for(i=0;i<pocet;i++) {
    v = a+i;
    printf("%2ld %2ld %10lf  %1ld\n",\
	   v->id,v->n,v->f,v->opt);
  }
  printf("\n");
  return;
}

/* Vypis Genu Populace */
GLOBAL void TSGprn(TSPop *a)
{
  WriteTabPop(a,stdout);
  WriteLinePop(a,stdout,a->ngen);
  return;
}

/***********************************************************************************************/
/* Fce pro vypis promennych i s odkazy */
/* Vypis nosniku */
GLOBAL void TSBallprn(TDimBeam *a)
{
  TDBprn(a);
  TSCprn(a->pc);
  printf("Momentova vyztuz :\n");
  TSSprn(a->ps);
  printf("Smykova vyztuz :\n");
  TSSprn(a->psw);
  TDBdimprn(a);
  return;
}

/* Vypis konstrukbe */
GLOBAL void TSKallprn(TSKce *a)
{
  int i;
  TDimBeam *pdb;

  TSKprn(a);
  for(i=0;i<a->nb;i++) {
    pdb = a->pdb+i;
    TSBallprn(pdb);
  }
  return;
}

/* Vypis Populace */
GLOBAL void TSPallprn(TSPop *a)
{
  TSPprn(a);
  TSVprn(a->pr,a->npr);
  TSGprn(a);
  return;
}

/***********************************************************************************************/
/* Fce na vypsani optimalizacnich hodnot */
GLOBAL void TDB_OPprn(TDimBeam *a)
{
  WriteCena(a,stdout);
  return;
}

/***********************************************************************************************/
/* Fce na vypsani matice */
GLOBAL void matprn(double* a,int n,int m)
{
  int i,j;

  for(i=0;i<n;i++) {
    for(j=0;j<m;j++) {
      printf(" %15lf",mat(a,i,j,m));
    }
    printf("\n");
  }
  printf("\n");
  return;
}

/***********************************************************************************************/
/* Fce pro tisk bitovych cisel */
GLOBAL void B_prn(bit co)
{
  int j;

  printf("Bit: ");
  for(j=31;j>-1;j--) {
    printf("%1ld",((co >> j) & 1));
  }
  printf("\n");
  return;
}
