/*
 * rc_mat.c		ver. 1.0
 *
 * Soubor vyuzivajici fce z rovnice.c pro reseni rovnic
 * od Karla Matouse
 * pridany maticove operace
 *
 * M. Leps, 26.8.1998
 *
 */

#include "rc.h"

/***********************************************************************************************/
/* Fce vynasobi matice a(m*n) * b(n*p)= c(m*p) , vse ulozeno v radcich */
GLOBAL void nasmat(double *a,double *b,double *c,int m,int n,int p)
{
  int i,j,k;

  nulmat(i,m*p,c);

  for(i=0;i<m;i++) {
	 for(k=0;k<p;k++) {
		for(j=0;j<n;j++) {
		  mat(c,i,k,p) += (mat(a,i,j,n)*mat(b,j,k,p));
		}
	 }
  }

  return;
}

/***********************************************************************************************/
/* procedura na reseni rovnic */
GLOBAL void reseni_rovnic(double *a,double *x,double *y,long n,long m,long as)
  
  /*	funkce provadi reseni maticove rovnice A.X=Y
		lze ji tedy bez problemu pouzit na vypocet inverzni matice
		matice A je plna
		
		A(n,n),X(n,m),Y(n,m)
		as - rozhodovaci konstanta
		as=1 - pivot se hleda jen v pripade, ze A(i,i)=0
		as=2 - pivot se hleda pokazde
		
		testovano 24.7.1996 pomoci programu test_fin_res.c v /u/jk/TESTY/METODY
		procedura dava stejne vysledky jako ldl,ldlkon,ldlblok,congrad_sky,congrad_comp
		
		**** otestovano ****
		*/
{		 
  long    i,j,k,ac,acr,acc,aca,aca1,acx,acy,acy1,aci,acj;
  long    *av;
  double  s,g;
  
  /*av = (long*) calloc (n,sizeof(long));*/
  av = new long[n] ;

  /*************************************************************************/
  /*  nastaveni hodnot vektoru, ktery udava poradi jednotlivych neznamych  */
  /*************************************************************************/
  for (i=0;i<n;i++){
	 av[i]=i;
  }
  
  for (i=0;i<n-1;i++){
	 acr=i;  acc=i;
	 if (as==1){
		/******************************************/
		/*  pivot se hleda jen pokud je A(i,i)=0  */
		/******************************************/
		if (fabs(a[i*n+i])<1.0e-30){
		  /*  vyber pivota  */
		  s=0.0;
		  /*  smycka pres radky  */
		  for (j=i;j<n;j++){
			 aca=j*n+i;
			 /*  smycka pres sloupce  */
			 for (k=i;k<n;k++){
				if (s<fabs(a[aca])){
				  s=fabs(a[aca]);  acr=j;  acc=k;
				}
				aca++;
			 }
		  }
		  if (s==0.0){
			 printf ("\n Singularni matice v procedure gemp (krok %ld).\n",i);
			 abort ();
		  }
		}
	 }
	 if (as==2){
		/****************************/
		/*  pivot se hleda pokazde  */
		/****************************/
		s=0.0;
		/*  smycka pres radky  */
		for (j=i;j<n;j++){
		  aca=j*n+i;
		  /*  smycka pres sloupce  */
		  for (k=i;k<n;k++){
			 if (s<fabs(a[aca])){
				s=fabs(a[aca]);  acr=j;  acc=k;
			 }
			 aca++;
		  }
		}
		if (s==0.0){
		  printf ("\n Singularni matice v procedure reseni_rovnic(krok %ld).\n",i);
		  abort ();
		}
	 }
	 
	 /******************/
	 /*  vymena radku  */
	 /******************/
	 if (acr!=i){
		aca=i*n+i;  aca1=acr*n+i;
		for (j=i;j<n;j++){
		  s=a[aca];
		  a[aca]=a[aca1];
		  a[aca1]=s;
		  aca++;  aca1++;
		}
		acy=i*m;  acy1=acr*m;
		for (j=0;j<m;j++){
		  s=y[acy];
		  y[acy]=y[acy1];
		  y[acy1]=s;
		  acy++;  acy1++;
		}
	 }
	 /********************/
	 /*  vymena sloupcu  */
	 /********************/
	 if (acc!=i){
		ac=av[i];
		av[i]=av[acc];
		av[acc]=ac;
		
		aca=i;  aca1=acc;
		for (j=0;j<n;j++){
		  s=a[aca];
		  a[aca]=a[aca1];
		  a[aca1]=s;
		  aca+=n;  aca1+=n;
		}				
	 }
	 /***************/
	 /*  eliminace  */
	 /***************/
	 
	 for (j=i+1;j<n;j++){
		acj=j*n+i;  aci=i*n+i;
		s=a[acj]/a[aci];
		/*  modifikace matice A  */
		for (k=i;k<n;k++){
		  a[acj]-=s*a[aci];
		  acj++;  aci++;
		}
		acj=j*m;  aci=i*m;
		/*  modifikace matice pravych stran Y  */
		for (k=0;k<m;k++){
		  y[acj]-=s*y[aci];
		  acj++;  aci++;
		}
	 }
  }
  
  /*****************/
  /*  zpetny chod  */
  /*****************/
  
  for (i=n-1;i>-1;i--){
	 g=a[i*n+i];  acx=i*m;
	 for (j=0;j<m;j++){
		s=0.0;  aca=i*n+i+1;  acy=(i+1)*m+j;
		for (k=i+1;k<n;k++){
		  s+=a[aca]*x[acy];
		  aca++;  acy+=m;
		}
		x[acx]=(y[acx]-s)/g;
		acx++;
	 }
  }
  
  /***********************************/
  /*  prerovnani do puvodniho stavu  */
  /***********************************/
  for (i=0;i<n;i++){
	 if (av[i]!=i){
		for (j=i;j<n;j++){
		  if (av[j]==i){
			 acc=j;  break;
		  }
		}
		
		ac=av[i];
		av[i]=av[acc];
		av[acc]=ac;
		
		aca=i*m;  aca1=acc*m;
		for (j=0;j<m;j++){
		  s=x[aca];
		  x[aca]=x[aca1];
		  x[aca1]=s;
		  aca++;  aca1++;
		}
	 }
  }
  
  /*free (av);*/
  delete [] av ;
}
