/*
 * rc_optim.c		ver. 1.02
 *
 * soubor s funkcemi pro optimalizaci
 *
 * M. Leps, 5.12.1998
 *
 */

#include "rc.h"

#define DILY3
//#define DELKY2

/* Deklarace LOCAL funkci */
LOCAL void rozdel(int all,int i1,int i2,int *c1,int *c2,int *c3);
LOCAL double PenFce(double Fu,double Fs,double delta, double beta );

/* upravy reseni namisto penalt */
LOCAL void pass_amin(TDimBeam *pdb, PopVar &pr ) ;
LOCAL void set_50( PopVar &pr ) ;

inline int getnas( int oi ) { int n ; ( oi==1 )?( n=2 ):( n=oi ) ; return( n ) ; } ;

/***********************************************************************************************/
/* Fce pro vypocet ojemu a ceny materialu */
GLOBAL void OP_DBcena(TDimBeam *pdb)
{
  int i;
  double sas,sac,vsw,lm,lw;
  TSBeam *pb;
  TSMdim *pm;
  TSWdim *pw;

  pb = pdb->pb;
  pdb->tp.vc = 0.0;
  pdb->tp.vs = 0.0;
  pdb->tp.vsw = 0.0;
  pdb->tp.wc = 0.0;
  pdb->tp.ws = 0.0;

  for(i=0;i<3;i++) {
	 pm = pdb->m+i;
	 pw = pdb->w+i;
	 lm = (pm->s)*(pb->pe->l);
	 lw = (pw->s)*(pb->pe->l);
	 sas = pm->as[0].a + pm->as[1].a;   /* celkova plocha vyztuze */
	 sac = ((pdb->b)*(pdb->h))-sas;     /* celkova plocha betonu */
	 vsw = (pdb->h+pdb->b-2*pdb->c[0]-2*pdb->c[1]+2*w_pr(pdb->prw)+0.1)\
		*(pw->pw)*npr_as(1,w_pr(pdb->prw))*lw/pw->sw;
	 pdb->tp.vsw += vsw;
	 pdb->tp.vc += sac*lm;
	 pdb->tp.vs += sas*lm;
  }

  pdb->tp.wc = pdb->tp.vc*2500.0;
  pdb->tp.ws = (pdb->tp.vs+pdb->tp.vsw)*7850.0;
  pdb->tp.cc = pdb->tp.vc*(pdb->pc->c1);
  pdb->tp.cs = pdb->tp.ws*(pdb->ps->c1);
  pdb->tp.c = pdb->tp.cc + pdb->tp.cs;
  return;
}

/***********************************************************************************************/
/* Fce pro prevod z binarniho modu */
GLOBAL void OP_bin_to_D(TDimBeam *pb, PopVar &pr /*TSPop *pp*/)
{
  pb->b = pr[0].r; /*0.15;*/
  pb->h = pr[1].r; /*0.400;*/

#define SVYZTUZI
#ifdef SVYZTUZI

  pb->pr[DOLNI] = pr[2].i;
  pb->pr[HORNI] = pr[3].i;
  pb->m[0].as[DOLNI].n = getnas( pr[4].i ) ;
  pb->m[1].as[DOLNI].n = getnas( pr[5].i ) ;
  pb->m[2].as[DOLNI].n = getnas( pr[6].i ) ;
  pb->m[0].as[HORNI].n = getnas( pr[7].i ) ;
  pb->m[1].as[HORNI].n = getnas( pr[8].i ) ;
  pb->m[2].as[HORNI].n = getnas( pr[9].i ) ;

#ifdef DILY3
	rozdel(pb->ne,pr[10].i,pr[11].i,&(pb->m[0].s),&(pb->m[1].s),&(pb->m[2].s));
#endif
#ifdef DELKY2
	rozdel(pb->ne,0,pr[11].i,&(pb->m[0].s),&(pb->m[1].s),&(pb->m[2].s));
#endif
#ifdef KONSTD
	pb->m[0].s=9 ;
	pb->m[1].s=3 ;
	pb->m[2].s=0 ;
	printf( "-- pozor, konstantni delky -- \n " ) ;
#endif

#else

/* zmena bez vyztuze !!!!!*/

  pb->m[0].s = 4 ;
  pb->m[1].s = 4 ;
  pb->m[2].s = 4 ;
  
  pb->pr[DOLNI] = 5 ;
  pb->pr[HORNI] = 5 ;
  pb->m[0].as[DOLNI].n = 3 ;
  pb->m[1].as[DOLNI].n = 3 ;
  pb->m[2].as[DOLNI].n = 2 ;
  pb->m[0].as[HORNI].n = 0 ;
  pb->m[1].as[HORNI].n = 0 ;
  pb->m[2].as[HORNI].n = 6 ;

#endif

#define SESMYKEM
#ifdef SESMYKEM

  pb->prw = pr[14].i;
	 pb->w[0].sw = pr[15].r;
	 pb->w[1].sw = pr[16].r;
	 pb->w[2].sw = pr[17].r;
	 pb->w[0].pw = pr[18].i;
	 pb->w[1].pw = pr[19].i;
	 pb->w[2].pw = pr[20].i;

  rozdel(pb->ne,pr[12].i,pr[13].i,&(pb->w[0].s),&(pb->w[1].s),&(pb->w[2].s));

#else
	
	/* zmena!!!!! bez smyku */

  pb->prw = 2 ;
  pb->w[0].sw = 0.2 ;
  pb->w[1].sw = 0.2 ;
  pb->w[2].sw = 0.125 ;
  pb->w[0].pw = 2 ;
  pb->w[1].pw = 2 ;       
  pb->w[2].pw = 2 ;

  pb->w[0].s = 4 ;
  pb->w[1].s = 4 ;
  pb->w[2].s = 4 ;

#endif


  pb->m[0].as[DOLNI].a = npr_as(pb->m[0].as[DOLNI].n,i_pr(pb->pr[DOLNI]));
  pb->m[0].as[HORNI].a = npr_as(pb->m[0].as[HORNI].n,i_pr(pb->pr[HORNI]));
  pb->m[1].as[DOLNI].a = npr_as(pb->m[1].as[DOLNI].n,i_pr(pb->pr[DOLNI]));
  pb->m[1].as[HORNI].a = npr_as(pb->m[1].as[HORNI].n,i_pr(pb->pr[HORNI]));
  pb->m[2].as[DOLNI].a = npr_as(pb->m[2].as[DOLNI].n,i_pr(pb->pr[DOLNI]));
  pb->m[2].as[HORNI].a = npr_as(pb->m[2].as[HORNI].n,i_pr(pb->pr[HORNI]));
	
  return;
}

/* Fce pro prevod do binarniho modu */
GLOBAL void OP_D_to_bin(TDimBeam *pb, PopVar &pr /*TSPop *pp*/)
{
  pr[0].r = pb->b;
  pr[1].r = pb->h;
  pr[2].i = pb->pr[DOLNI];
  pr[3].i = pb->pr[HORNI];
  pr[4].i = pb->m[0].as[DOLNI].n;
  pr[5].i = pb->m[1].as[DOLNI].n;
  pr[6].i = pb->m[2].as[DOLNI].n;
  pr[7].i = pb->m[0].as[HORNI].n;
  pr[8].i = pb->m[1].as[HORNI].n;
  pr[9].i = pb->m[2].as[HORNI].n;
  pr[10].i = pb->m[0].s;
  pr[11].i = pb->m[1].s;

  //smyk
	pr[12].i = pb->w[0].s;
	pr[13].i = pb->w[1].s;
	pr[14].i = pb->prw;
	pr[15].r = pb->w[0].sw;
	pr[16].r = pb->w[1].sw;
	pr[17].r = pb->w[2].sw;
  pr[18].i = pb->w[0].pw;
	pr[19].i = pb->w[1].pw;
	pr[20].i = pb->w[2].pw;

  return;
}

/* Fce pro prechod od dim k GA */
GLOBAL void OP_oldtonew(TDimBeam *yng,TDimBeam *old, PopVar &p )
{
  /* predani jinych hodnot z puvodni kce */
  yng->id = old->id;
  yng->ne = old->ne;
  yng->l = old->l;
  yng->ps = old->ps;
  yng->psw = old->psw;
  yng->pc = old->pc;
  yng->alfa = old->alfa;
  yng->pb = old->pb;

	/* upravy reseni */
//	set_50( p ) ;

  /*OP_update(p,old->gen);*/ /* neoptimalizovane hodnoty */
  OP_bin_to_D(yng,p);              /* hodnoty z GA */ 
  FEtoDim(yng,yng->pb);            /* vnitrni sily */

	/* upravy reseni */
	pass_amin( yng, p ) ;

  DimCheck(yng);                   /* kontrola RC */

  OP_DBcena(yng);                  /* vypocet ceny Dim */
}

/* prevzeti neoptimalizovanych hodnot */
GLOBAL void OP_update(TSPop *pb, PopVar &pr /*TSGen *gkce*/)
{
/*  pr[12].i = pb->w[0].s;
	 pr[13].i = pb->w[1].s;
	 pr[14].i = pb->prw;
	 pr[15].d = pb->w[0].sw;
	 pr[16].d = pb->w[1].sw;
	 pr[17].d = pb->w[2].sw;
	 pr[18].i = pb->w[0].pw;
	 pr[19].i = pb->w[1].pw;
	 pr[20].i = pb->w[2].pw;*/

/*  int i;
	 for(i=0;i<p->npr;i++) {
      if(p->pr[i].opt == 0)
	 p->a->bin[i] = gkce->bin[i];
	 }*/
  return;
}

/***********************************************************************************************/
/* Fce vyhodnocujici silu populace */
GLOBAL void OP_silaPop(TSPop *p,TDimBeam *pdb,int akce)
{
  int i;

  if(akce == NEWPOP) {
    for(i=0;i<p->ngen;i++) {
      seta(p,i);
/*      OP_silaGen(p,pdb);*/
    }
  }
  else {
    seta(p,p->ngen+0);
/*    OP_silaGen(p,pdb);*/
    seta(p,p->ngen+1);
/*    OP_silaGen(p,pdb);*/
  }
  return;
}

/***********************************************************************************************/
/* Fce vyhodnocujici silu genu po prepoctu tuhosti */
GLOBAL void OP_newObjFnc( PopVar &p,TDimBeam *pdb )
{
  double penalty ;
  penalty = 0.0;

	/* upravy reseni */
	set_50( p ) ;

  OP_bin_to_D(pdb,p);              /* hodnoty z GA */ 
  FEtoDim(pdb,pdb->pb);            /* vnitrni sily */

	/* upravy reseni */
	pass_amin( pdb, p ) ;

  DimCheck(pdb);                   /* kontrola RC */

  OP_DBcena(pdb);                  /* vypocet ceny Dim */

  p.cena=pdb->tp.c ;               /* cena do Genu */

  /* podminky */
  for( int i=0 ; i<3 ; i++ ) {
#ifdef __FUNKCE_H__
		GlobalFunkce.add_penalty(pdb->m[i].m[0]/pdb->m[i].mu[0], p ) ;
		GlobalFunkce.add_penalty(pdb->m[i].m[1]/pdb->m[i].mu[1], p ) ;
		GlobalFunkce.add_penalty(pdb->w[i].q/pdb->w[i].qu, p ) ;
#else
	if(pdb->m[i].m[0] != 0.0) {
	  penalty += PenFce(pdb->m[i].m[0],pdb->m[i].mu[0], 0.02, 2.0 ) ;
	}
	if(pdb->m[i].m[1] != 0.0) {
	  penalty += PenFce(pdb->m[i].m[1],pdb->m[i].mu[1], 0.02, 2.0 ) ;
	}
	if(pdb->w[i].q != 0.0) {
	  penalty += PenFce(pdb->w[i].q,pdb->w[i].qu, 0.02, 2.0 ) ;
	}
#endif
  }

  p.net_f=p.cena/1002.65 ;

#ifndef __FUNKCE_H__
  p.f=p.net_f+penalty ;
  p.chi=penalty ;
#endif

  return;
}
/***********************************************************************************************/
/* Fce vyhodnocujici silu genu */
GLOBAL void OP_silaGen(  PopVar &p,TDimBeam *pdb)
{
  double penalty ;
  penalty = 0.0;

  TDimBeam bnew;

  nullTDB(&bnew);

  /* predani jinych hodnot z puvodni kce */
  OP_oldtonew(&bnew,pdb,p);

  p.cena=bnew.tp.c ;               /* cena do Genu */

  /* podminky */
  for( int i=0 ; i<3 ; i++ ) {
#ifdef __FUNKCE_H__
		GlobalFunkce.add_penalty(bnew.m[i].m[0]/bnew.m[i].mu[0], p ) ;
		GlobalFunkce.add_penalty(bnew.m[i].m[1]/bnew.m[i].mu[1], p ) ;
		GlobalFunkce.add_penalty(bnew.w[i].q/bnew.w[i].qu, p ) ;
#else
	if(bnew.m[i].m[0] != 0.0) {
	  penalty += PenFce(bnew.m[i].m[0],bnew.m[i].mu[0], 0.02, 2.0 ) ;
	}
	if(bnew.m[i].m[1] != 0.0) {
	  penalty += PenFce(bnew.m[i].m[1],bnew.m[i].mu[1], 0.02, 2.0 ) ;
	}
	if(bnew.w[i].q != 0.0) {
	  penalty += PenFce(bnew.w[i].q,bnew.w[i].qu, 0.02, 2.0 ) ;
	}
#endif
  }

  /*  min = ((pdb->b-0.05))*((pdb->h)-0.05)*(pdb->l)*(pdb->pc->c1);*/
  double min = 0.;

  //p.net_f=p.cena/1002.65 ;
  p.net_f=(p.cena-min)/(pdb->tp.c-min);
#ifndef __FUNKCE_H__
  //p.f=p.cena ;
  p.f=p.net_f+penalty ;
  p.chi=penalty ;
#endif
  return;
}

LOCAL double PenFce( double Fs, double Fu, double delta, double beta )
{
  double chi ;
  const double badmax=20. ;

/*  if( Fu<=0. ) */          /* nulovy nebo zaporny Fu */
/*    return( badmax ) ;*/

  chi=Fs/Fu ;
  if( chi<=1. ) /* Fs pod Fu, tedy good */
    return( 0.0 ) ;
  else {  /* Fs nad Fu */
    if(( chi-1.)<=delta )
      return( pow(( chi-1.)/delta , beta )) ;
    else
      return( badmax ) ;/* veliky Fs */
  }
}

/***********************************************************************************************/
/* pokud nevyhovi test na a_min, upravi d a prislusny chromozom */
LOCAL void pass_amin(TDimBeam *pdb, PopVar &pr )
{
  int i,j;
  double *as;
	double all_as ;
  TSMdim *pe;

  pdb->c[0] = vypocc(SMRAZEM,pdb->pr[0],pdb->prw);
  pdb->c[1] = vypocc(SMRAZEM,pdb->pr[1],pdb->prw);

  for(i=0;i<3;i++) {
		pe = &(pdb->m[i]);
		for(j=0;j<2;j++){
			if(pe->m[j] == 0.0)
				continue;
			as = &(pe->as[j].a);
			/* kontrola MSU */
			checknav(pe->hodn[j],pe->pch[j],&(pe->d[j]),pe->m[j],&(pe->mu[j]),*as,pdb->b,pdb->h,pdb->c[j],pe->as[j].n,i_pr(pdb->pr[j]),\
					pdb->ps->fyk,pdb->ps->fyd,pdb->pc->fck,pdb->pc->fcd);
		}
		/* kontrola stupne vyztuzeni */
		all_as=pe->as[0].a+pe->as[1].a ;
		j=checkv(all_as,pdb->b,pdb->h,pe->d[0], pdb->ps->fyk); // nepresne
		if( j==ERRLOWS ) {
			pdb->h=round( (all_as*pdb->ps->fyk/(600.*pdb->b))+pdb->c[0] , 0.001 );
			pr[1].r=pdb->h ;
			pr.change=BYes ;
		}
	}
}

/***********************************************************************************************/
/* zaruci 50% vyztuze u dolnich vlaken */
LOCAL void set_50( PopVar &pr )
{
	int most ;

#ifdef DILY3
	//printf( "-- pozor, 3dily -- \n " ) ;

	most=pr[4].i ;
	if( most<pr[5].i )
		most=pr[5].i ;
	if( most<pr[6].i )
		most=pr[6].i ;

	most=(int)ceil( 0.5*(double)most ) ;

	if( most>pr[4].i ) {
		pr[4].i=most ;
		pr.change=BYes ;
	}
	if( most>pr[5].i ) {
		pr[5].i=most ;
		pr.change=BYes ;
	}
	if( most>pr[6].i ) {
		pr[6].i=most ;
		pr.change=BYes ;
	}

#endif
#ifdef DELKY2
	//printf( "-- pozor, 2dily -- \n " ) ;

	most=pr[5].i ;
	if( most<pr[6].i )
		most=pr[6].i ;

	most=(int)ceil( 0.5*(double)most ) ;

	if( most>pr[5].i ) {
		pr[5].i=most ;
		pr.change=BYes ;
	}
	if( most>pr[6].i ) {
		pr[6].i=most ;
		pr.change=BYes ;
	}

#endif
}

/***********************************************************************************************/
/* Inicializace prom. pro RC */
GLOBAL TSPop* OP_initVect(int pocgen,char *jm,int reseni)
{
  int i;
  TSPop* pop;
  FILE *fp;
  if ((fp = fopen(jm, "r")) == NULL) {
	 printf("Soubor %s se nepodarilo otevrit \n",jm);
	 abort();
  }
  //InitRandom();
  printf("-- Pozor, zmena kodu, neprobehla inicializace populace -- \n") ;
  abort() ;
  //pop = NewPop(21,pocgen);
  pop->reseni = reseni;
  pop->a = pop->pg;

  /* b;h;pr dolni;pr horni;dolni n1,n2,n3;horni n1,n2,n3; */
  /* l1;l2;lw1;lw2;prw;sw1;sw2;sw3;pw1;pw2;pw3 */
  for(i=0;i<21;i++) {
	 fscanf(fp,"%ld %ld %c %lf %ld",&(pop->pr[i].id),\
			  &(pop->pr[i].n),&(pop->pr[i].t),&(pop->pr[i].f),&(pop->pr[i].opt));
  }

  if (fclose(fp) == EOF) {
	 printf("Soubor optprom se nepodarilo uzavrit \n");
	 uvolniTSP(pop);
	 abort();
  }
  return(pop);
}

/***********************************************************************************************/
/* Rozdeli elementy */
LOCAL void rozdel(int all,int i1,int i2,int *c1,int *c2,int *c3)
{
  int r;
  r = all - i1;
  if(r <= 0) {
    *c1 = all;
    *c2 = 0;
    *c3 = 0;}
  else {
    *c1 = i1;
    if(r < i2) {
      *c2 = r;
      *c3 = 0;}
    else {
      *c2 = i2;
      *c3 = r-i2;}}
  return;
}
