#include <stdio.h>
#include <math.h>
#include "neuronka.h"

#define LAYERS 3
#define LAY1   4
#define LAY2   6
#define LAY3   1
#define ALPHA  0.5


#define TrainSet  LAY1
#define TrainMin  0.1
#define TrainMax  0.5
#define TrainRange 0.005
#define TrainStep ( TrainRange / TrainSet )

#define TestSet   20

double source[LAY1+1] , target[LAY3+1] ; // global
p_double oO ;

double func ( double &opar )
{
    //    return 1/ ( 1+ exp (  - ( 1/ ( 1 + exp ( -opar ) ) ) ) );
    //return 0.4 ;
    return (0.2*sin(20*opar)*opar)+0.5 ;
    //if ( opar > 0.3 ) return (0.2*sin(20*opar)*opar)+0.5 ;
    //return (0.2*sin(20*opar)*opar)+0.6 ;
    //return 0.39*opar+0.5;
    //return exp ( opar )/4 -1.0 ;
}

double func2 ( double ox, double oy )
{
    return ox+oy;
}

neuronka::neuronka ( void ) : objective_function ()
{
    int i;

    layers=0 ;
    lunits=NULL ;
    nunits=0 ;
    last=0 ;
    nerror=0 ;
    index=NULL ;
    output=NULL ;
    layers_prepare ( LAYERS ) ;
    layers_add ( LAY1 ) ;
    layers_add ( LAY2 ) ;
    layers_add ( LAY3 ) ;
    initialize ( ALPHA ) ;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0]=-10.0;
        Domain[i][1]=10.0;
        Domain[i][2]=0.0;
    }
    optimum = new double;
    *optimum = 0.0;
    precision = 0.0000001;
    sprintf ( name, "neuronka" );
    outfile = new char [256];
    outfile = &(name[0]);
}

neuronka::~neuronka ( void )
{
}

void neuronka::layers_prepare ( int on )
{
    lunits=new int[on] ;
    if ( !lunits ) errmsg ( "layers_prepare ()" ) ;
    index=new int[on] ;  // nebo kontrola v layers_add, bude asi stacit o jedno min
    if ( !index ) errmsg ( "layers_prepare ()" ) ;
    index[0]=0 ;
}

void neuronka::layers_add ( int on )   // pridat kontrolu na pridavani
{                                 // mozna lunits + bunits
    nunits+=on+1 ;
    lunits[layers]=on+1 ;         // +1 for bias
    layers++ ;
}

void neuronka::initialize ( double oalpha )
{
    alpha=oalpha;
    last=layers-1 ;

    int i , tmp ;
    for( i=1 ; i<layers ; i++ )
    {
        tmp=lunits[i-1]*lunits[i] ;
        index[i]=index[i-1]+tmp ;
        Dim+=tmp ;
    }
    
    alloc () ;
}

void neuronka::finalize ( void )
{
}

void neuronka::alloc ( void )
{
    output=new p_double[layers] ;

    if ( !output ) errmsg ( "alloc ()" ) ; // tohle predelat
    int i ;
    for( i=1 ; i<layers ; i++ ) {
        output[i]=new double[lunits[i]] ;
        if ( !( output [i] ) ) errmsg ( "alloc ()" ) ;
        output[i][0]=1.0 ;
    }
}

double neuronka::activate ( double &osum )
{
    return 1.0/( 1.0+exp( -alpha*osum )) ;
}


int neuronka::getindex ( int &olayer , int &oprev , int &othis ) // inline
{
    return index[olayer-1]+oprev*lunits[olayer]+othis ;
}

void neuronka::propagate ( void )
{
    int i, j, k ;
    for ( i=1 ; i<layers ; i++ ) // optimize
        for ( j=1 ; j<lunits[i] ; j++ )
        {
            double sum=0.0 ;
            for ( k=0 ; k<lunits[i-1] ; k++ )
                sum+=weight[ getindex ( i , k , j ) ]*output[i-1][k] ;
            output[i][j]=activate ( sum ) ;
        }
}

void neuronka::output_error ( p_double otarget )
{
    int i ;
    double err ;
    nerror=0.0 ;
    for ( i=1 ; i<lunits[last] ; i++ )
    {
        err=otarget[i]-output[last][i] ;
        nerror+=sqr ( err ) ;
    }
    nerror*=0.5 ;
}

void neuronka::evaluate (  double *oCH )
{
    weight = oCH ;
    int j ;
    double startx , starty, x = TrainMin ;
    output[0]=source ;
    oO=output[last] ;

    FILE *f;
    f=fopen ( "log", "wt" );

    do
    {
        startx=x ;
        starty=x ;
        for( j=1 ; j<LAY1+1 ; j++ )
        {
            source[j]=func2 ( startx, starty ) ;
            startx+=TrainStep ;
            starty+=TrainStep ;
        }

        propagate ();


        //printf( "%0.10f %0.10f\n" , start , oO[1] ) ;
        fprintf( f, "%0.10f %0.10f %0.10f\n" , startx , starty, oO[1] ) ;
        x+=0.0001 ;
    }
    while ( x < 0.9 ) ;

    fclose ( f );
}

void neuronka::test ( p_double oI , p_double otarget )
{
    output[0]=oI ;
    propagate ();
    output_error ( otarget ) ;
}

double neuronka::value ( double *oCH )
{
    weight = oCH ;
    int i , j ;
    double err ;
    err=0 ;
    for( i=0 ; i<TestSet ; i++ )
    {
        /*
        double start = random_double ( TrainMin , TrainMax ) ;
        for( j=1 ; j<LAY1+1 ; j++ )
        {
            source[j]=func ( start ) ; start+=TrainStep ;
        }
        target[1]=func ( start );
        */
        double x = random_double ( TrainMin , TrainMax ) ;
        double y = random_double ( TrainMin , TrainMax ) ;
        for( j=1 ; j<LAY1+1 ; j++ )
        {
            source[j]=func2 ( x, y ) ;
            x+=TrainStep ;
            y+=TrainStep ;
        }
        target[1]=func2 ( x, y );
        test ( source , target ) ;
        err+=nerror ;
    }
    err/=TestSet ;
    return -err ;
}

