#include "puc.h"
#include <math.h>

int CompPoint ( const void *p_a, const void *p_b)
{
  POINT *f_a, *f_b;   

  f_a = (POINT *) p_a; 
  f_b = (POINT *) p_b;
  
  if ( f_a->y > f_b->y)
    return(1);
  else 
    if (f_a->y < f_b->y)
      return(-1);
    else
      if (f_a->x > f_b->x)
	return(1);
      else
	if (f_a->x < f_b->x)
	  return(-1);
	else
	  return(0);
 
}

puc::puc ( void ) : objective_function ()
{
    double phi;  /* Volume fraction of fibers */

    FILE *fr;
    unsigned i;

    /* Allocate global variables */

    MALLOC (par, 1, KPARAMS );
    par->fib = NULL; par->r_meas = NULL;

    /* Read input file */

    FOPEN (fr, "puc/input.txt", "r");

    fscanf (fr, "%d", &(par->num_fibers)); /* Read number of fibers */

    MALLOC(par->fib, par->num_fibers, POINT);

    /* Read position of fiber centers */

    for ( i = 0 ; i < par->num_fibers; i++ )
        fscanf (fr, "%lf %lf %lf", &(par->r), &(par->fib[i].x), &(par->fib[i].y) );
  
    par->r *= .5;

    FCLOSE( fr );

    /* Compute values of K function for reference particle configuration */

    par->P_max.x = P_maxx;
    par->P_max.y = P_maxy;

    /* Sampling template */

    par->num_meas = num_templ;

    MALLOC (par->r_meas, par->num_meas, double);

    for (i = 0 ; i < par->num_meas; i++)
        par->r_meas[i]=10.*par->r/par->num_meas*(i+1);

    MALLOC (K_ref, par->num_meas, double);

    /* Now compute reference function values */

    K( K_ref, par );

    /* Compute volume fraction of fibers */

    phi = PI*SQR(par->r)*par->num_fibers/(par->P_max.x * par->P_max.y );

    par->num_fibers = num_newfib;
    /* Determine dimension of RVE for new number of fibers */
    par->P_max.x = par->P_max.y = sqrt( par->num_fibers*PI*SQR(par->r)/phi );

    /* (De)allocate appropriate arrays */

    FREE(par->fib);

    MALLOC ( par->fib, par->num_fibers, POINT );
    MALLOC ( K_akt, par->num_meas, double );

    Dim=20;
    Domain = new p_double [Dim];
    for ( i=0; i<Dim; i++ )
    {
        Domain[i] = new double [3];
        Domain[i][0] = 0.0;
        Domain[i][1] = par->P_max.x;
        Domain[i][2] = 0.0;
    }
    optimum = new double;
    *optimum = -0.00005899;
    precision = 0.0000001;
    sprintf ( name, "puc" );
    outfile = new char [256];
    outfile = &(name[0]);
    Return_to_domain = 1;
}

puc::~puc ( void )
{
    FREE ( par->fib );
    FREE ( par->r_meas );
    FREE ( par );
    FREE ( K_ref );
    FREE ( K_akt );
}

double puc::value (double* oCH)
{
  double t,u; 
  int i;
  
  for ( i = 0 ; i < par->num_fibers; i++ )     
    {
      par->fib[i].x = oCH[2*i];
      par->fib[i].y = oCH[2*i+1];
    }
  
  qsort ( par->fib, par->num_fibers, sizeof(POINT), &CompPoint ); /* Sort fibers with respect to y coordinate */

  K( K_akt, par );
  
  for ( i = 0, t = .0 ; i < par->num_meas ; i++) 
    { 
      u = ( K_akt[i] - K_ref[i] )/( PI*SQR(par->r_meas[i]) ) ;
      t += (u*u);
    }

  return(-t);
}
