#include "K.h"

#define SQR(a) ((a)*(a))

static long NumPoints( const POINT A,  const POINT B,  const POINT P_search,
		       double r_search, const KPARAMS *params );

void K ( double *k, KPARAMS *params )
{
  int i,j;
  double r_akt, t;
  long I_akt;
  POINT P_akt,P_a,P_b;

  t = (double) (params->P_max.x * params->P_max.y/(params->num_fibers*
						   params->num_fibers));

  for( i = 0; i < params->num_meas; i++)
    {
      I_akt = 0;
      r_akt = params->r_meas[i];
      
      for( j = 0; j < params->num_fibers; j++)
	{
	  P_akt = params->fib[j];
	  P_a.x = P_akt.x-r_akt;
	  P_a.y = P_akt.y-r_akt;

	  P_b.x = P_akt.x+r_akt;
	  P_b.y = P_akt.y+r_akt;
      
	  I_akt += NumPoints(P_a,P_b,P_akt,r_akt, params)-1;
	}

      k[i]= I_akt*t;
    }

  return;
}

static long NumPoints( const POINT A,  const POINT B,  const POINT P_search,
		       double r_search, const KPARAMS *params )
{
  int i_x, i_x_min, i_x_max; /* x indexes */
  int i_y, i_y_min, i_y_max; /* y indexes */

  double x_min,x_max, y_min, y_max;
  int y_start, y_end;
  long n_found;

  double t, u;

  POINT P_act, P_t;
  int j,k;

  i_x_min = (int) (A.x/params->P_max.x);
  if (A.x < .0)
    i_x_min--;
  i_x_max = (int) (B.x/params->P_max.x);
  
  i_y_min = (int) (A.y/params->P_max.y);
  if (A.y < 0.)
    i_y_min--;

  i_y_max = (int) (B.y/params->P_max.y);

  n_found = 0;
  r_search*= r_search;
 
  for ( i_y = i_y_min; i_y <= i_y_max; i_y++) 
    {

      t = i_y *params->P_max.y;
      y_min = (t > A.y ) ? 0 : A.y - t;

      t += params->P_max.y;
      y_max = ( t < B.y ) ? params->P_max.y : params->P_max.y - t + B.y;

      P_act.y = P_search.y - i_y*params->P_max.y;

      for ( i_x = i_x_min; i_x <= i_x_max; i_x++) 
	{
	
	  /* Determine searching window */
	  t = i_x*params->P_max.x;
	  x_min = ( t > A.x ) ? 0 : A.x - t;

	  t += params->P_max.x;
	  x_max = (t < B.x ) ? params->P_max.x : params->P_max.x - t + B.x;
      
	  /* Actual position of reference point */
	  P_act.x = P_search.x - i_x*params->P_max.x;
	
	  /* Start search */

	  /* Lower index */

	  y_start = 0;

	  if ( y_min >= params->fib[0].y) 
	    {
	      j = params->num_fibers - 1;
	      
	      while( (j-y_start) > 2 )
		{
		  k= (j+y_start)/2;
    
		  if( params->fib[k].y < y_min) y_start = k;
		  else j = k;
		}
	    }

	  y_end = (params->num_fibers - 1);

	  if ( y_max <= params->fib[params->num_fibers-1].y ) 
	    {
	      j = y_start;
 
	      while( (y_end-j)>2 )
		{
		  k = (j+y_end)/2;

		  if(params->fib[k].y > y_max) y_end = k;
		  else j = k;
		}
	    }
	
	  for( j = y_start; j <= y_end; j++)
	    {
	      P_t = params->fib[j];

	      if ((P_t.x >= x_min) && (P_t.x <= x_max)) 
		{
		  u =  SQR(P_act.x-P_t.x);
		  u += SQR(P_act.y-P_t.y);
		
		  if( u <= r_search ) n_found++;	  
		}
	    }
	}
    }

  return( n_found );
}








