/****************************************************
*                                                   *               
* FILE: 	==== Share.h ====                          *   
*                                                   *
* HEADER FILE FOR Share.c                           *
* Contains files shared by all programs             *
*                                                   *
* Authors:           Jan Zeman, Michal Sejnoha      *
* Latest update:     February 13, 2000              *
*                                                   *
*****************************************************/

#ifndef _SHARE_
#define _SHARE_

/* Default header files */

#include <stdio.h>
#include <stdlib.h>

/* Symbolic constants */

#define TRUE 1
#define FALSE 0
#define OR ||
#define AND &&

#define PI 3.141592
#define SQR(a) ((a)*(a))

/* allocation error */

#define AllocError(var) fprintf(stderr,"Cannot allocate %s in file %s line %d, stack's capacity exceeded\n", #var, __FILE__, __LINE__ );exit(1);

#define FileError(fmode,name) fprintf(stderr,"Cannot %s file %s in file %s line %d\n", fmode, name, __FILE__,__LINE__);exit(2);

#define NotNULLError(var) fprintf(stderr,"You tried to allocate not NULL variable %s in file %s line %d\n",#var,__FILE__,__LINE__);  

/*
#define NotNULLError(var)
*/
/* Memory manipulating macros */
#define MALLOC(var,number,type) if (var != NULL){NotNULLError(var)} else if( (var = (type*) malloc( (number)*sizeof(type) ) ) == NULL ){AllocError(var)}
#define FREE(var) if((var) != NULL) {free((void *) var); var = NULL;}

/* "Vector" memory allocation */
#define CreateVector(var,number,type) if (var != NULL) {NotNULLError(var)} else if ( (var = (type *) calloc((number),sizeof(type))) == NULL ){AllocError(var)}
#define FreeVector(var) if ((var) != NULL) {free( (void*) var); var=NULL;} 

/* "Matrix" memory allocation */
#define CreateMatrix(name,rows,columns,type) {int _ii_; for(_ii_=0; _ii_<rows;_ii_++){ CreateVector(name[_ii_],columns,type)}}

#define FreeMatrix(name,rows) {int _ii_; for( _ii_=0; _ii_<rows;_ii_++) FreeVector(name[_ii_]);}
/* File manipulating macros */

/* "Matrix" memory allocation - pure dynamic */
#define CreateMatrix_dyn(name,rows,columns,type) {int _ii_; if (name !=NULL) {NotNULLError(name)} else \
{MALLOC(name,rows,type*); for(_ii_=0; _ii_<rows;_ii_++) {name[_ii_]=NULL;CreateVector(name[_ii_],columns,type)}}}

#define FOPEN(file, name, mode) if( ( file = fopen(name,mode) ) == NULL) {FileError("open", name)}
#define FCLOSE(file) if (( fclose(file) ) == EOF) {FileError("close","")}

#define Error(string,value) fprintf(stderr,string,value);exit(1)

/* Procedures */  

extern void FreeProblem(void);
extern void PrintError(int numerror);
#define PError(num) {printf("\nError in file %s line %d.\n",__FILE__, __LINE__);PrintError(num);}

#endif


