#if !defined ( __GARTOU__ )
#include "gartou.h"
#endif

#include <stdio.h>

void gartou::init_vilog ( void )
{
    if ( !vilog ) return;

#if defined ( __HAVE_KRESLITKO__ )
    running=0;
    open_graphics ();
#endif
}

void gartou::close_vilog ( void )
{
    if ( !vilog ) return;

#if defined ( __HAVE_KRESLITKO__ )
    close_graphics() ;
#endif
}

int gartou::vilog_news ( int ceraf )
{
    if ( !vilog ) return 1;

    static double sbsf=MinDouble;

    if ( !ceraf ) printf ( "CERAF\n" );

    if ( (bsf_value-sbsf) > F->precision )
    {
        if ( F->optimum )
        {
            printf( "fc=%d   BTG=%10g   opt=%.8f   extremes:%d\n", fitness_call, btg_value, *F->optimum, n_extreme ) ;
        }
        else printf( "fc=%d   BTG=%.8f   extremes:%d\n", fitness_call, btg_value, n_extreme ) ;
        printf( "killed:%d\n", killed );
    }

    if ( bsf_value>sbsf )
    {
        //printf ( "***new BSF!!!!!" );
        sbsf=bsf_value;
    }

#if defined ( __HAVE_KRESLITKO__ )
    Ko->delete_items( 1 ) ;
    Ko->delete_items( 11 ) ;
    Ko->redraw_items( 3 ) ;
    draw_chromos( CH,ActualSize ) ;
    draw_cerafs ( all_locations, all_radius, n_extreme );
    if ( !until_next_step () ) return 0;
#endif

    return 1;
}

void gartou::vilog_results ()
{
    if ( !vilog ) return;

    int i;

    if ( F->optimum )
    {
        printf( "fc=%d   BSF=%.8f   opt=%.8f\n", fitness_call, bsf_value, *F->optimum ) ;
    }
    else printf( "fc=%d   BSF=%.8f\n", fitness_call, bsf_value ) ;

    printf ( "x=( %.8f",bsf[0] );
    for ( i=1; i<F->Dim; i++ ) printf ( ", %.8f", bsf[i] );
    printf ( " )\n" );
}

void gartou::vilog_make_out ()
{
    if ( !vilog ) return;

    int i,n;
    char file[20];

    FILE *f;

    if ( F->outfile )
    {
        sprintf ( file, "%s.out", F->outfile );
        f = fopen ( file, "at" );
    }
    else f = fopen ( "results", "at" );

    fprintf ( f, "%d\t%.5g\t%d\t", fitness_call, bsf_value, n_extreme );
    if ( F->optimum ) fprintf ( f, "%.5g\t", fabs(*F->optimum-bsf_value) );
    for ( i=0; i<F->Dim; i++ ) fprintf ( f, "%.5g\t", bsf[i] );
    fprintf ( f, "\n" );
    fclose ( f );
}

#if defined ( __HAVE_KRESLITKO__ )
void gartou::open_graphics ( void )
{
    Ko=new kreslitko ;
    Ko->create_button( "Close",CloseSignal ) ;
    Ko->create_button( "Next",NextSignal ) ;
    Ko->create_button( "Start/Stop",StartStopSignal ) ;
    Ko->open_graphics( F->Domain[_X][0],F->Domain[_Y][0],F->Domain[_X][1],F->Domain[_Y][1],400,400 ) ;
    Ko->set_background_color( 150,130,100 ) ;
    Ko->assign_rgb_color( 1,0,0,150 ) ;
    Ko->assign_rgb_color( 2,0,100,0 ) ;
    Ko->assign_rgb_color( 3,200,0,0 ) ;
    Ko->set_color( 1 ) ;
    Ko->set_text_color( 3 ) ;
    Ko->assign_font( 0,12,"Times",0 ) ;
    Ko->x_scale( F->Domain[_Y][0],F->Domain[_X][0],F->Domain[_X][1],0,8 ) ;
    Ko->y_scale( F->Domain[_X][0],F->Domain[_Y][0],F->Domain[_Y][1],0,8 ) ;
}

void gartou::draw_chromos ( double **och , int on )
{
    double xave=0.0,yave=0.0 ;
    int i ;

    Ko->set_color( 1 ) ;
    
    Ko->begin_serial() ;
    for ( i=0 ; i<on ; i++ )
    {
        Ko->mark_percentual( och[i][_X],och[i][_Y],KRESLITKO_MarkCross,3,1 ) ;
        xave+=och[i][_X] ;
        yave+=och[i][_Y] ;
    }
    xave/=( double )on ;
    yave/=( double )on ;
    Ko->mark_percentual( xave,yave,KRESLITKO_MarkRectangle,15,1 ) ;
    Ko->flush_serial() ;
    Ko->is_ready() ;
}

void gartou::draw_cerafs ( double **olocations, double **oradius, int on )
{
    int i;

    Ko->set_color( 3 ) ;
    Ko->set_pen_width ( 3 );
    
    Ko->begin_serial() ;
    for ( i=0 ; i<on ; i++ )
    {
        Ko->ellipse ( olocations[i][_X], olocations[i][_Y], oradius[i][_X]/2, oradius[i][_Y]/2,11 );
    }
    Ko->flush_serial() ;
    Ko->is_ready() ;
    Ko->set_pen_width ( 1 );
}

int gartou::until_next_step ( void )
{
    int signal ;
    if ( running )
    {
        if ( !Ko->receive_signal( signal )) return 1 ;
        if ( signal==StartStopSignal ) running=0 ;
    }
    if ( !running )
    {
        Ko->wait_and_receive_signal( signal ) ;
        switch ( signal )
        {
            case CloseSignal:
                return 0 ;
                break ;
            case NextSignal:
                return 1 ;
                break ;
            case StartStopSignal:
                running=1 ;
                break ;
        }
    }
    return 1 ;
}

void gartou::close_graphics ( void )
{
    int signal ;
    do
    {
        Ko->wait_and_receive_signal( signal ) ;
    }
    while ( signal!=CloseSignal ) ;
    if ( Ko ) delete Ko ;
}
#endif

